"""

Python Desktop Server - blogger API

Copyright (c) 2002, Georg Bauer <gb@murphy.bofh.ms>

Permission is hereby granted, free of charge, to any person obtaining a copy of 
this software and associated documentation files (the "Software"), to deal in 
the Software without restriction, including without limitation the rights to 
use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies of
 
the Software, and to permit persons to whom the Software is furnished to do so, 
subject to the following conditions:

The above copyright notice and this permission notice shall be included in all 
copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR 
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS
 
FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR 
COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER 
IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN 
CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.

"""

# $Id: BloggerAPI.py,v 1.6 2004/01/28 14:32:13 gb Exp $

import sys
import time

import PyDS.Tool

class BloggerAPI(PyDS.Tool.StandardTool):

	# this method creates a new posting. the appkey is ignored, as is the blogId.
	# categories can't be accessed via this call, use the metaWeblogAPI for that.
	# publish directly accesses the onhome flag, so postings that shouldn't publish
	# are marked as such. username and password are taken from the RPC credentials.
	# postings are html if they start with a tag, and structured text otherwise!
	def newPost(self, appkey, blogId, username, password, content, publish):
		prefs = self.getPreferencesTool()
		weblog = self.getToolByNamespace('weblog')
		if username != prefs.get('RPC', 'username') or password != prefs.get('RPC', 'password'):
			raise _('Authentication failure')
		structured = 2
		if content[0] == '<':
			structured = 0
		id = weblog.postNewEntry('', '', content, '', '', publish, structured=structured)
		return id
	
	# this method edits the text content of a post and enables/disables publishing.
	# appkey is ignored. rules are the same as with newPost.
	def editPost(self, appkey, postId, username, password, content, publish):
		prefs = self.getPreferencesTool()
		weblog = self.getToolByNamespace('weblog')
		if username != prefs.get('RPC', 'username') or password != prefs.get('RPC', 'password'):
			raise _('Authentication failure')
		structured = 2
		if content[0] == '<':
			structured = 0
		weblog.editOldEntry(postId, text=content, onhome=publish, structured=structured)
		return 1
	
	# this method returns just a list with one item, "default", as actually we don't support
	# different blog IDs
	def getUsersBlogs(self, appkey, username, password):
		prefs = self.getPreferencesTool()
		upstream = self.getToolByNamespace('upstream')
		if username != prefs.get('RPC', 'username') or password != prefs.get('RPC', 'password'):
			raise _('Authentication failure')
		return [{
			'blogid': 'weblog',
			'url': upstream.get('prefs', 'cloudurl'),
			'blogName': prefs.get('private', 'title')
		}]
	
	# this method returns a users information record, simulated from preferences
	def getUserInfo(self, appkey, username, password):
		prefs = self.getPreferencesTool()
		upstream = self.getToolByNamespace('upstream')
		if username != prefs.get('RPC', 'username') or password != prefs.get('RPC', 'password'):
			raise _('Authentication failure')
		(firstname, lastname) = prefs.get('private', 'name').split(' ', 1)
		return {
			'userid': username,
			'firstname': firstname,
			'lastname': lastname,
			'nickname': username,
			'email': prefs.get('private', 'email'),
			'url': upstream.get('prefs', 'cloudurl')
		}
	
	# dummy method, as templates are not supported via blogger API
	def getTemplate(self, appkey, username, password, templateType):
		prefs = self.getPreferencesTool()
		if username != prefs.get('RPC', 'username') or password != prefs.get('RPC', 'password'):
			raise _('Authentication failure')
		return ''
	
	# dummy method, as templates are not supported via blogger API
	def setTemplate(self, appkey, username, password, template, templateType):
		prefs = self.getPreferencesTool()
		if username != prefs.get('RPC', 'username') or password != prefs.get('RPC', 'password'):
			raise _('Authentication failure')
		return 1
	
	# this method fetches a posting from the database. appkey, as always, is ignored.
	# most fields of the struct returned are simulated, dateCreated is in the standard
	# C time format (since it isn't specified precisely in the blogger API).
	# This is an extension taken from Conversant!
	def getPost(self, appkey, postId, username, password):
		prefs = self.getPreferencesTool()
		weblog = self.getToolByNamespace('weblog')
		if username != prefs.get('RPC', 'username') or password != prefs.get('RPC', 'password'):
			raise _('Authentication failure')
		post = weblog.getPost(postId)
		if post:
			return {
				'content': post['text'],
				'userId': username,
				'postId': postId,
				'dateCreated': time.strftime(time.localtime(post['pubtime']))
			}
		else:
			raise _('Posting %s not found') % postid
	
	# this method returns the list of recent posts that show up on your homepage.
	# same rules as above, appkey and blogId are ignored.
	# This is an extension taken from Conversant!
	def getRecentPosts(self, appkey, blogId, username, password, numberOfPosts):
		prefs = self.getPreferencesTool()
		weblog = self.getToolByNamespace('weblog')
		if username != prefs.get('RPC', 'username') or password != prefs.get('RPC', 'password'):
			raise _('Authentication failure')
		liste = []
		for post in weblog.getRecentPosts(maxnumber=numberOfPosts):
			liste.append({
				'content': post['text'],
				'userId': username,
				'postId': post['id'],
				'dateCreated': time.strftime(time.localtime(post['pubtime']))
			})
		return liste
	
	# this method deletes a posting from the database. Rules are as with previous calls
	# the publish is ignored (that's how it is speced - stupid, but true)
	# This is an extension taken from Conversant!
	def deletePost(self, appkey, postId, username, password, publish):
		prefs = self.getPreferencesTool()
		weblog = self.getToolByNamespace('weblog')
		if username != prefs.get('RPC', 'username') or password != prefs.get('RPC', 'password'):
			raise _('Authentication failure')
		weblog.deleteOldEntry(postId)
		return 1
	
__desc__ = _("This tool implements the blogger API.")

PyDS.Tool.registerTool(BloggerAPI, 'blogger', _('Blogger API'), __desc__, 1410)

