"""

Python Desktop Server - Calendar management functions

Copyright (c) 2002, Georg Bauer <gb@murphy.bofh.ms>

Permission is hereby granted, free of charge, to any person obtaining a copy of 
this software and associated documentation files (the "Software"), to deal in 
the Software without restriction, including without limitation the rights to 
use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies of
 
the Software, and to permit persons to whom the Software is furnished to do so, 
subject to the following conditions:

The above copyright notice and this permission notice shall be included in all 
copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR 
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS
 
FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR 
COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER 
IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN 
CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.

"""

# $Id: CalendarTool.py,v 1.27 2004/09/27 09:35:31 gb Exp $

import time
import string

try:
	from cPickle import loads, dumps
except ImportError:
	from pickle import loads, dumps

import PyDS.Tool

import calendar
calendar.setfirstweekday(_PyDS.firstweekday)
wdnames = _('Mo Tu We Th Fr Sa Su').split()
wdnames = wdnames[_PyDS.firstweekday:] + wdnames[:_PyDS.firstweekday]
mlnames = _('January February March April May June July August September October November December').split()
msnames = _('Jan Feb Mar Apr May Jun Jul Aug Sep Oct Nov Dec').split()

class DayMonthCache:

	def __init__(self, tool, category):
		self._days = {}
		self._months = {}
		self._tool = tool
		self._category = category
		for (y, m, d) in tool.getItemDays(category):
			k = y * 100 + m
			if self._months.has_key(k):
				self._months[k][d] = 1
			else:
				self._months[k] = {int(d): 1}
		self._monthlist = self._months.keys()
		self._monthlist.sort()
	
	def __getstate__(self):
		return (
			self._days,
			self._months,
			self._category,
			self._tool.name
		)
	
	def __setstate__(self, obj):
		self._days = obj[0]
		self._months = obj[1]
		self._category = obj[2]
		self._tool = PyDS.Tool.getTool(obj[3])
		self._monthlist = self._months.keys()
		self._monthlist.sort()
	
	def recache(self, year, month, day):
		k = year * 100 + month
		monthlistchanged = 0
		hasitems = self._tool.hasItemsForDay(year, month, day, self._category)
		if self._months.has_key(k):
			if self._months[k].has_key(day):
				del self._months[k][day]
				if self._months == {}:
					if hasitems:
						self._months[k][day] = 1
					else:
						del self._months[k]
						monthlistchanged = 1
				else:
					if hasitems:
						self._months[k][day] = 1
			else:
				if hasitems:
					self._months[k][day] = 1
		else:
			if hasitems:
				self._months[k] = {day:1}
				monthlistchanged = 1
		if monthlistchanged:
			self._monthlist = self._months.keys()
			self._monthlist.sort()
		
	def hasDay(self, year, month, day):
		k = year * 100 + month
		if self._months.has_key(k):
			return self._months[k].has_key(day)
		else:
			return 0
		
	def hasUptoMonth(self, year, month):
		if len(self._monthlist):
			k = year * 100 + month
			if self._monthlist[0] <= k:
				return reduce(max, [i for i in self._monthlist if i <= k])
			else:
				return 0
		else:
			return 0
		
	def hasBeyondMonth(self, year, month):
		if len(self._monthlist):
			k = year * 100 + month
			if self._monthlist[-1] >= k:
				return reduce(min, [i for i in self._monthlist if i >= k])
			else:
				return 0
		else:
			return 0
			
	def render(self):
		src = []
		for mspec in self._monthlist:
			year = int(mspec/100)
			month = mspec - year*100
			daylist = self._months[mspec].keys()
			daylist.sort()
			daylist = map(str, daylist)
			src.append('%04d-%02d(%s)' % (year, month, string.join(daylist, ',')))
		return string.join(src, '<br>')

class CalendarTool(PyDS.Tool.StandardTool):

	def _createdb(klass, db):
		db.getas("toolcache[tool:S,pickle:B]")

	_createdb = classmethod(_createdb)
	
	def _remove(self):
		PyDS.Tool.StandardTool._remove(self)
		del self._cache

	def _initdb(self):
		try:
			self._acquire()
			self.cache = self.db.view('toolcache').ordered(1)
			self._cache = {}
			for cache in self.cache:
				if PyDS.Tool.hasTool(cache.tool):
					self._cache[cache.tool] = loads(cache.pickle)
			self.importantTool = 1
		finally: self._release()
	
	def _initactive(self):
		timer = self.getToolByNamespace('timer')
		timer._addHook('select', 'calendar', _('drop calendar cache for tool'), self._calendarTimerHook)
	
	def _calendarTimerHook(self, qual, item=None):
		if qual == 'title':
			return _('drop calendar cache for tool')
		elif qual == 'select':
			html = '<p><select name="item">'
			liste = self._cache.keys()
			liste.sort()
			for toolname in liste:
				html += '<option value="%s">%s' % (
					toolname,
					toolname
				)
			html += '</select>'
			return html
		elif qual == 'item':
			return _('dropping the calendar cache for tool "%s"') % item
		elif qual == 'run':
			self.dropCache(item)
		else:
			raise ValueError('%s is an unknown qualifier' % qual)

	def _shutdown(self):
		if _PyDS.verbose: print _('syncing calendar caches in shutdown')
		for tool in self._cache.keys():
			self._syncCache(tool)
	
	def _syncCache(self, toolname):
		try:
			self._acquire()
			self.logVerbose(_('syncing the calendar cache for tool %s') % toolname)
			if _PyDS.verbose: print _(' ... for tool %s') % toolname
			if self._cache.has_key(toolname):
				(idx, found) = self.cache.locate({'tool':toolname})
				if found:
					cache = self.cache[idx]
					cache.pickle = dumps(self._cache[toolname])
				else:
					cache = dumps(self._cache[toolname])
					self.cache.append({
						'tool':toolname,
						'pickle':cache
					})
				self._commit()
		finally: self._release()
		
	def dropCache(self, toolname):
		try:
			self._acquire()
			self.logVerbose(_('dropping the calendar cache for tool %s') % toolname)
			if self._cache.has_key(toolname):
				for cat in self._cache[toolname].keys():
					self._cache[toolname][cat] = None
				del self._cache[toolname]
			(idx, found) = self.cache.locate({'tool':toolname})
			if found:
				self.cache.delete(idx)
			self._commit()
		finally: self._release()
		
	def _findCache(self, tool, category):
		toolname = tool.name
		if self._cache.get(toolname,{}).has_key(category):
			return self._cache[toolname][category]
		else:
			try:
				self._acquire()
				if not self._cache.has_key(toolname):
					self._cache[toolname] = {}
				self._cache[toolname][category] = DayMonthCache(tool, category)
				return self._cache[toolname][category]
			finally: self._release()
			
	def _hasItemsForDay(self,tool, year, month, day, category):
		return self._findCache(tool, category).hasDay(year, month, day)
		
	def _hasItemsUpto(self, tool, year, month, category):
		res = self._findCache(tool, category).hasUptoMonth(year, month)
		if res:
			return (int(res / 100), int(res % 100), 1)
		else:
			return (year, month, 0)
		
	def _hasItemsBeyond(self, tool, year, month, category):
		res = self._findCache(tool, category).hasBeyondMonth(year, month)
		if res:
			return (int(res / 100), int(res % 100), 1)
		else:
			return (year, month, 0)

	# This method renders a calendar for use in templates
	def render(self, tool, withtoday=0, urlcallback=None, year=None, month=None, day=None, onhome=None, category=None):
		(tyear, tmonth, tday) = (time.localtime(time.time()))[:3]
		if type(tool) == type(''):
			tool = self.getToolByNamespace(tool)
		if urlcallback == None:
			urlcallback = tool.getStaticUrl
		if year == None and month == None and day == None:
			(year, month, day) = (tyear, tmonth, tday)
		html = '<table border=0>'
		html += '<tr><th class="smallfont" colspan=7>%s %d</th></tr>' % (mlnames[month-1], year)
		html += '<tr>'
		for col in wdnames:
			html += '<th class="smallfont" align="right">%s</th>' % col
		html += '</tr>'
		for row in calendar.monthcalendar(year, month):
			html += '<tr>'
			for col in row:
				if col:
					url = urlcallback(year=year, month=month, day=col, category=category)
					if self._hasItemsForDay(tool, year, month, col, category):
						if year == tyear and month == tmonth and col == tday and withtoday:
							html += '<td class="smallfont" align="right"><a href="%s"><b>%2d</b></a></td>' % (url, col)
						else:
							html += '<td class="smallfont" align="right"><a href="%s">%2d</a></td>' % (url, col)
					else:
						if year == tyear and month == tmonth and col == tday and withtoday:
							html += '<td class="smallfont" align="right"><b>%2d</b></td>' % col
						else:
							html += '<td class="smallfont" align="right">%2d</td>' % col
				else:
					html += '<td class="smallfont" align="right">&nbsp;</td>'
			html += '</tr>'
		mp = month - 1
		yp = year
		if mp == 0:
			mp = 12
			yp -= 1
		mn = month + 1
		yn = year
		if mn == 13:
			mn = 1
			yn += 1
		html += '<tr>'
		(yp, mp, found) = self._hasItemsUpto(tool, yp, mp, category)
		if found:
			url = urlcallback(year=yp, month=mp, category=category)
			html += '<th class="smallfont" colspan=2 align="left"><a href="%s">%s<br>%d</a></th>' % (url, msnames[mp-1], yp)
		else:
			html += '<th class="smallfont" colspan=2 align="left">%s<br>%d</th>' % (msnames[mp-1], yp)
		html += '<td class="smallfont" colspan=3 align="center">'
		if (day == tday and month == tmonth and year == tyear) or not(withtoday):
			html += '&nbsp;'
		else:
			html += '<a href="%s">%s</a>' % (
				urlcallback(year=tyear, month=tmonth, day=tday, category=category),
				_('Today')
			)
		html += '</td>'
		(yn, mn, found) = self._hasItemsBeyond(tool, yn, mn, category)
		if found:
			url = urlcallback(year=yn, month=mn, category=category)
			html += '<th class="smallfont" colspan=2 align="right"><a href="%s">%s<br>%d</a></th>' % (url, msnames[mn-1], yn)
		else:
			html += '<th class="smallfont" colspan=2 align="right">%s<br>%d</th>' % (msnames[mn-1], yn)
		html += '</tr>'
		html += '</table>'
		return html

	def index_html(self, req):
		html = _("<p>Check those tools whose calendar cache should be rebuilt")
		html += '<form action="%s" method="POST">' % self.getUrl('recache_redir')
		html += '<table border=0>'
		liste = self._cache.keys()
		liste.sort()
		for toolname in liste:
			tool = self.getToolByNamespace(toolname)
			html += '<tr><td><input type="checkbox" name="tool" value="%s"></td><td><b>%s:</b> %s</td></tr>' % (
				toolname,
				'<a href="%s">%s</a>' % (
					self.getUrl('cache_html', tn=toolname),
					toolname
				),
				tool.title
			)
		html += "</table>"
		html += '<input type=submit name="CacheRebuild" value="%s">' % _('rebuild')
		html += '<input type=submit name="CacheSync" value="%s">' % _('sync')
		html += "</form>"
		req.setLocalValue('title', _('Manage Calendar Cache'))
		return req.renderPage('BaseTemplate', body=html)
		
	def cache_html(self, req):
		tn = req.getFirstValue('tn')
		html = '<dl>'
		liste = self._cache[tn].keys()
		liste.sort()
		for cat in liste:
			catname = cat
			if cat == None:
				catname = _('all items')
			elif cat == '':
				catname = _('default category')
			html += '<dt><b>%s</b></dt><dd>%s</dd>' % (
				catname,
				self._cache[tn][cat].render()
			)
		html += "</dl>"
		req.setLocalValue('title', _("Cache for tool %s") % tn)
		return req.renderPage('BaseTemplate', body=html)
	
	def recache_redir(self, req):
		cachesync = req.getFirstValue('CacheSync')
		cacherebuild = req.getFirstValue('CacheRebuild')
		for tool in req.getAllValues('tool'):
			if cacherebuild:
				self.dropCache(tool)
			if cachesync:
				self._syncCache(tool)
		return req.getUrl()

	# recache a tools cache for a category and a given reference time or direct date
	# either reftime or year+month+day must be given. category spec is either None
	# (for all items) or a given category name ('' might be a default category)
	def recache(self, tool, category=None, year=None, month=None, day=None, reftime=None):
		if reftime:
			(year, month, day) = (time.localtime(reftime))[:3]
		if type(tool) == type(''):
			tool = self.getToolByNamespace(tool)
		cache = self._findCache(tool, category)
		try:
			self._acquire()
			cache.recache(year, month, day)
		finally: self._release()
		return 1
		
		
__desc__ = _("This tool manages a calendar and it's rendering.")

PyDS.Tool.registerTool(CalendarTool, 'calendar', _('Calendar'), __desc__, 920)
