"""

Python Desktop Server - dynamic Context (fluid-let, flet) for python threads

Copyright (c) 2002, Georg Bauer <gb@murphy.bofh.ms>

Permission is hereby granted, free of charge, to any person obtaining a copy of 
this software and associated documentation files (the "Software"), to deal in 
the Software without restriction, including without limitation the rights to 
use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies of
 
the Software, and to permit persons to whom the Software is furnished to do so, 
subject to the following conditions:

The above copyright notice and this permission notice shall be included in all 
copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR 
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS
 
FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR 
COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER 
IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN 
CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.

"""

# $Id: Context.py,v 1.11 2004/07/06 08:36:52 gb Exp $

import threading

from copy import copy

_ContextStack = {}
_Context = {}

class Context:

	def __init__(self):
		# set up the default dynamic namespace
		self.begin(
			cloud=1,
			desktop=0,
			rpc=0
		)
	
	def clone(self, newThread):
		thread = threading.currentThread()
		_ContextStack[newThread] = []
		_Context[newThread] = copy(_Context[thread])
	
	def begin(self, **kw):
		thread = threading.currentThread()
		if not(_ContextStack.has_key(thread)):
			_ContextStack[thread] = []
		if not(_Context.has_key(thread)):
			_Context[thread] = {}
		_ContextStack[thread].append(copy(_Context[thread]))
		for k in kw.keys():
			_Context[thread][k] = kw[k]
	
	def beginDesktop(self, **kw):
		if not(kw.has_key('desktop')): kw['desktop'] = 1
		if not(kw.has_key('cloud')): kw['cloud'] = 0
		apply(self.begin, [], kw)
	
	def beginCloud(self, **kw):
		if not(kw.has_key('desktop')): kw['desktop'] = 0
		if not(kw.has_key('cloud')): kw['cloud'] = 1
		apply(self.begin, [], kw)

	def end(self):
		thread = threading.currentThread()
		_Context[thread] = _ContextStack[thread].pop()

	def has(self, var):
		thread = threading.currentThread()
		return _Context[thread].has_key(var)
	
	def get(self, var, default=None):
		if self.has(var):
			return _Context[threading.currentThread()][var]
		else:
			return default

	def __getattr__(self, var):
		try:
			return _Context[threading.currentThread()][var]
		except KeyError:
			raise AttributeError(var)
	
	def __setattr__(self, var, val):
		_Context[threading.currentThread()][var] = val

	def __delattr__(self, var):
		del _Context[threading.currentThread()][var]
	
	def __nonzero__(self):
		return 1

	def __hash__(self):
		return id(self)

	def keys(self): 
		return _Context[threading.currentThread()].keys()

	def get(self, key, default=None): 
		"Dict-style retrieval by key with default."
		return _Context[threading.currentThread()].get(key, default)
	
	def getContext(self):
		"returns a shallow copy of the current context for later use"
		return copy(_Context[threading.currentThread()])
	
	def newContext(self, context):
		"pushes a new context on top"
		self.begin(**context)
	
# patch the single context instance into the global namespace
import __builtin__
__builtin__.__dict__['_flet'] = Context()

