"""

Python Desktop Server - Documentation of Class APIs (for Tools)

Copyright (c) 2002, Georg Bauer <gb@murphy.bofh.ms>

Permission is hereby granted, free of charge, to any person obtaining a copy of 
this software and associated documentation files (the "Software"), to deal in 
the Software without restriction, including without limitation the rights to 
use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies of
 
the Software, and to permit persons to whom the Software is furnished to do so, 
subject to the following conditions:

The above copyright notice and this permission notice shall be included in all 
copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR 
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS
 
FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR 
COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER 
IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN 
CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.

"""

# $Id: DocumentationTool.py,v 1.10 2004/01/28 14:32:13 gb Exp $

import pydoc
import re

import PyDS.Tool

patchre = re.compile('Data and non-method functions defined here.*(Methods inherited from)', re.DOTALL)

class MyHTMLDoc(pydoc.HTMLDoc):

	def classlink(self, object, modname):
		return pydoc.classname(object, modname)

	def section(self, title, fgcol, bgcol, contents, width=10, prelude='', marginalia=None, gap='&nbsp;&nbsp;'):
		result = '<h2>%s</h2>' % title
		result += '<p>%s</p>' % contents
		return result

	def docroutine(self, object, name=None, mod=None, funcs={}, classes={}, methods={}, cl=None):
		if name.find('_') >= 0:
			return ''
		elif name in ('match', 'error', 'getPreferencePanels',
			'needsStuff'):
			return ''
		else:
			return pydoc.HTMLDoc.docroutine(self, object, name, mod, funcs, classes, methods, cl)

# To make a standard tool, inherit from StandardTool
class DocumentationTool(PyDS.Tool.StandardTool):

	def _initopts(self):
		try:
			self._acquire()
			self.doc = MyHTMLDoc()
			self.hasAPI = 0
		finally: self._release()

	def index_html(self, req):
		html = "<ul>"
		for tool in PyDS.Tool.getTools():
			if tool.hasAPI:
				url = self.getUrl('doc_html', name=tool.name)
				html += '<li>%s</li>' % tool.nugget(url)
		html += "</ul>"
		req.setLocalValue('title', _("Tools with an API"))
		return req.renderPage('BaseTemplate', body=html)

	def doc_html(self, req):
		tool = self.getToolByNamespace(req.getFirstValue('name'))
		html = self.doc.document(tool.__class__)
		html = patchre.sub(lambda m: m.group(1), html)
		return req.renderPage('BaseTemplate', body=html, withcalendar=0)
		
__desc__ = _("This is a tool that dynamically creates class documentation.")

PyDS.Tool.registerTool(DocumentationTool, 'documentation', _('Documentation'), __desc__, 895)

