"""

Python Desktop Server - Driver Registry

Copyright (c) 2002, Georg Bauer <gb@murphy.bofh.ms>

Permission is hereby granted, free of charge, to any person obtaining a copy of 
this software and associated documentation files (the "Software"), to deal in 
the Software without restriction, including without limitation the rights to 
use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies of
 
the Software, and to permit persons to whom the Software is furnished to do so, 
subject to the following conditions:

The above copyright notice and this permission notice shall be included in all 
copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR 
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS
 
FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR 
COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER 
IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN 
CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.

"""

# $Id: DriverRegistry.py,v 1.3 2003/12/18 10:38:44 gb Exp $

import metakit

# Registration of installed upstream drivers
_UpstreamDrivers = []

# register a driver
def registerDriver(tag, description, klass):
	_UpstreamDrivers.append((tag, description, klass))

# get a list of installed drivers
def getDrivers():
	return _UpstreamDrivers

# get a list of installed drivers (only tag and description)
def getDriverSelection():
	liste = []
	for driver in _UpstreamDrivers:
		liste.append((driver[0], driver[1]))
	return liste

# get an instance of a driver
def getDriver(tag, tool):
	for driver in _UpstreamDrivers:
		if driver[0] == tag:
			print _('Initializing upstream driver %s') % driver[1]
			return driver[2](tool, driver[1])
	raise ValueError('invalid driver tag: %s' % tag)

# get a dummy xmlrpc result (like XSS, but faked)
def getDummyResult(flError=0, message='Ok.', **kw):
	res = kw
	res['flError'] = flError
	res['message'] = message
	return res

class BaseDriver:

	def __init__(self, tool, title):
		self.tool = tool
		self.title = title
	
	def initdb(self):
		self.tool.driver = metakit.view()

	def initopts(self):
		pass
	
	def _backup(self):
		return []

