"""

Python Desktop Server - static handler with link rewriting

Copyright (c) 2002, Georg Bauer <gb@murphy.bofh.ms>

Permission is hereby granted, free of charge, to any person obtaining a copy of 
this software and associated documentation files (the "Software"), to deal in 
the Software without restriction, including without limitation the rights to 
use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies of
 
the Software, and to permit persons to whom the Software is furnished to do so, 
subject to the following conditions:

The above copyright notice and this permission notice shall be included in all 
copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR 
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS
 
FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR 
COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER 
IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN 
CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.

"""

# '$Id: LinkRewritingHandler.py,v 1.8 2004/01/28 13:04:05 gb Exp $

import re

import PyDS.Tool

from PyDS.StaticHandler import static_handler

class link_rewriting_file_producer:

	out_buffer_size = 1<<15

	isXML = re.compile(r'^\s*<')

	def __init__(self, file):
		self.done = 0
		self.missing = 0
		self.file = file
		baseurl = PyDS.Tool.getTool('upstream').get('prefs', 'cloudurl')
		if baseurl and baseurl[-1] == '/':
			baseurl = baseurl[:-1]
			self.rx = re.compile(r'"%s(/.*?)"' % baseurl)
		self._webbug = PyDS.Tool.getTool('macros').webbug()
		self._commentsScript = PyDS.Tool.getTool('macros').commentsScript()
		self._trackbackScript = PyDS.Tool.getTool('macros').trackbackScript()
		self.data = self.file.read(self.out_buffer_size)
		if baseurl and self.isXML.match(self.data):
			self.rewrite = 1
		else:
			self.rewrite = 0
		self.fill()
	
	def fill(self):
		if self.rewrite:
			char = ' '
			while char and char != '>':	
				char = self.file.read(1)
				self.data += char
			l = len(self.data)
			self.data = self.rx.sub(lambda m: '"%spreview%s"' % (PyDS.Tool.pathprefix(), m.group(1)), self.data)
			if self._webbug:
				self.data = self.data.replace(self._webbug, '')
			if self._trackbackScript:
				self.data = self.data.replace(self._trackbackScript, '<script language="JavaScript" type="text/javascript">function trackbackCounter(n){document.write("?");return;}</script>')
			if self._commentsScript:
				self.data = self.data.replace(self._commentsScript, '<script language="JavaScript" type="text/javascript">function commentCounter(n){document.write("?");return;}</script>')
			self.missing += (l - len(self.data))
	
	def more(self):
		if self.done:
			return ''
		else:
			if not self.data:
				self.file.close()
				del self.file
				self.done = 1
				return ' ' * self.missing
			else:
				data = self.data
				self.data = self.file.read(self.out_buffer_size)
				self.fill()
				return data

class link_rewriting_handler(PyDS.StaticHandler.static_handler):

    default_file_producer = link_rewriting_file_producer

    useAuthentication = 1
    needAdmin = 0

