"""

Python Desktop Server - Passwordstorage (to prevent storage in macros!)

Copyright (c) 2002, Georg Bauer <gb@murphy.bofh.ms>

Permission is hereby granted, free of charge, to any person obtaining a copy of 
this software and associated documentation files (the "Software"), to deal in 
the Software without restriction, including without limitation the rights to 
use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies of
 
the Software, and to permit persons to whom the Software is furnished to do so, 
subject to the following conditions:

The above copyright notice and this permission notice shall be included in all 
copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR 
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS
 
FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR 
COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER 
IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN 
CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.

"""

# $Id: PasswordTool.py,v 1.7 2004/02/04 09:56:39 gb Exp $

import PyDS.Tool
import metakit

class PasswordTool(PyDS.Tool.StandardTool):

	def _createdb(klass, db):
		db.getas("passwords[title:S,username:S,password:S]")

	_createdb = classmethod(_createdb)

	def _initdb(self):
		try:
			self._acquire()
			self.passwords = self.db.view('passwords').ordered(1)
			self.importantTool = 1
		finally: self._release()
	
	def _passwordAsHash(self, password):
		return {
			'title': password.title,
			'username': password.username,
			'password': password.password
		}

	# this method returns a record for a password. If a property is given,
	# only that property is returned.
	def getPassword(self, title, property=None):
		try:
			self._acquire()
			(idx, found) = self.passwords.locate({'title': title})
			if found:
				p = self.passwords[idx]
				if property:
					if hasattr(p, property):
						return getattr(p, property)
					else:
						return ''
				else:
					return self._passwordAsHash(p)
			else:
				if property:
					return ''
				else:
					return {}
		finally: self._release()

	def _getPasswords(self):
		try:
			self._acquire()
			liste = []
			for p in self.passwords:
				liste.append(self._passwordAsHash(p))
			return liste
		finally: self._release()
	
	# this method returns description of preferences panels that are
	# hooked up into preferences by this tool
	def getPreferencePanels(self):
		return [
			['password_panel', _('Passwords'),
				_('You can configure passwords for your macros or other functions here.')]
		]
			
	def password_panel(self, req):
		title = req.getFirstValue('title')
		username = ''
		password = ''
		if title:
			username = self.getPassword(title, 'username')
			password = self.getPassword(title, 'password')
		html = '<p>%s' % _("This is a safe place for your usual passwords.")
		html += '<form action="%s" method="POST">' % req.getUrl('password_redir', tool='passwords')
		html += '<table border=0 width="100%">'
		for p in self._getPasswords():
			html += '<tr><td class="whitebox">'
			html += '<table border="0" width="100%"><tr>'
			html += '<td><b>%s:</b> %s/%s</td>' % (
				p['title'],
				p['username'],
				p['password']
			)
			html += '<td align="right">'
			html += '<a href="%s">%s</a>' % (
				self.getUrl('panel_html',
					tool='preferences',
					panel='password_panel',
					fortool='passwords',
					title=p['title']
				),
				_('edit'),
			)
			html += '<br><a href="%s">%s</a>' % (
				self.getUrl('delete_redir'),
				_('delete')
			)
			html += '</td></tr></table>'
			html += '</td></tr>'
		html += '</table>'
		html += '<table border="0">'
		html += '<tr><th>%s</th><td>' % _('title')
		html += '<input name="title" size=40 value="%s">' % title
		html += '</td></tr><tr><th>%s</th><td>' % _('username')
		html += '<input name="username" size=40 value="%s">' % username
		html += '</td></tr><tr><th>%s</th><td>' % _('password')
		html += '<input name="password" size=40 value="%s">' % password
		html += '</td></tr></table>'
		html += '<p align="right"><input type="submit" name="AddSubmit" value="%s">' % _('add')
		html += '</form>'
		return html

	def password_redir(self, req):
		try:
			self._acquire()
			title = req.getFirstValue('title')
			username = req.getFirstValue('username')
			password = req.getFirstValue('password')
			if title and username and password:
				(idx, found) = self.passwords.locate({'title': title})
				if found:
					p = self.passwords[idx]
					p.title = title
					p.username = username
					p.password = password
				else:
					self.passwords.append({
						'title': title,
						'username': username,
						'password': password
					})
				self._commit()
			return self.getUrl(
				'panel_html',
				tool='preferences',
				panel='password_panel',
				fortool='passwords'
			)
		finally: self._release()
	
	def delete_redir(self, req):
		try:
			self._acquire()
			title = req.getFirstValue('title')
			if title:
				(idx, found) = self.passwords.locate({'title': title})
				if found:
					self.passwords.delete(idx)
					self._commit()
		finally: self._release()

__desc__ = _('This is a safe place for passwords to be used by other tools. Safe because it is never exported to your cloud.')
	
PyDS.Tool.registerTool(PasswordTool, 'passwords', _('Passwords'), __desc__, 1110)

