"""

Python Desktop Server - Background Renderer Tool

Copyright (c) 2002, Georg Bauer <gb@murphy.bofh.ms>

Permission is hereby granted, free of charge, to any person obtaining a copy of 
this software and associated documentation files (the "Software"), to deal in 
the Software without restriction, including without limitation the rights to 
use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies of
 
the Software, and to permit persons to whom the Software is furnished to do so, 
subject to the following conditions:

The above copyright notice and this permission notice shall be included in all 
copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR 
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS
 
FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR 
COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER 
IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN 
CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.

"""

# $Id: RenderTool.py,v 1.8 2004/02/09 16:43:30 gb Exp $

import time
import string
import PyDS.Tool

class MultipleRenderQueue:

	def queues(self):
		l = []
		for tool in PyDS.Tool.getToolsWithMethod('_renderIndexHook'):
			l.append(tool.name)
		l.sort()
		return l
	
	def getqueue(self, tool):
		return PyDS.Tool.getTool(tool).queue
	
	def haswork(self):
		for tool in self.queues():
			if self.getqueue(tool).length():
				return tool
		return None

class RenderTool(PyDS.Tool.StandardTool):

	# some stuff is initialized in _initdb, because that is called
	# before all _initopts
	def _initdb(self):
		self._multiq = MultipleRenderQueue()
		self.importantTool = 1

	def _initopts(self):
		self.hasAPI = 0

	def _initthread(self, server):
		while server.isRunning():
			time.sleep(10)
			toolname = self._multiq.haswork()
			if toolname:
				while toolname:
					if _PyDS.verbose: print _('rendering requests from %s') % toolname
					queue = self._multiq.getqueue(toolname)
					if queue.length():
						while queue.length():
							job = queue.shift()
							jobf = job[0]
							if type(jobf) == type(''):
								tool = self.getToolByNamespace(toolname)
								jobf = getattr(tool, job[0])
							jobf(job[1], job[2])
						tool = self.getToolByNamespace(toolname)
						getattr(tool, '_renderIndexHook')()
					else:
						if _PyDS.verbose: print _('weird, no requests found for %s!') % toolname
					toolname = self._multiq.haswork()
				self.logVerbose(_('work done, going to sleep again'))
	
	def _status(self):
		try:
			self._acquire()
			html = PyDS.Tool.StandardTool._status(self)
			html += '<ul>'
			if self._multiq.haswork():
				html += _('<li>There are items in the render queue</li>')
				html += '<ul>'
				for k in self._multiq.queues():
					html += _('<li>There are %d items in the queue for tool %s</li>') % (self._multiq.getqueue(k).length(), k)
				html += '</ul>'
			else:
				html += _('<li>There are no items in the render queue (tools: %s)</li>') % string.join(self._multiq.queues(), ', ')
			html += self._threadstatus()
			html += '</ul>'
			return html
		finally: self._release()
	
	def _hookTool(self, tool, indexfunction):
		self._multiq.addqueue(tool)
		if type(indexfunction) == type(''):
			indexfunction = getattr(tool, indexfunction)
		self.indexf[tool.name] = indexfunction

		
__desc__ = _("This is a tool that manages abstract render queues for rendering other tools in the background.")

PyDS.Tool.registerTool(RenderTool, 'render', _('Render'), __desc__, 1200)

