"""

Python Desktop Server - main server module

Copyright (c) 2002, Georg Bauer <gb@murphy.bofh.ms>

Permission is hereby granted, free of charge, to any person obtaining a copy of 
this software and associated documentation files (the "Software"), to deal in 
the Software without restriction, including without limitation the rights to 
use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies of
the Software, and to permit persons to whom the Software is furnished to do so, 
subject to the following conditions:

The above copyright notice and this permission notice shall be included in all 
copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR 
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS
 
FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR 
COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER 
IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN 
CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.

"""

# $Id: Server.py,v 1.89 2004/09/28 07:02:25 gb Exp $

import os
import re
import sys
import glob
import shutil
import base64
import time
import string
import threading
import traceback
import PyDS.timeoutsocket
from PyDS.Daemonize import become_daemon
import asyncore
from medusa import http_server, status_handler, logger, filesys, monitor
import PyDS.select_trigger

import PyDS.Tool
import PyDS.StaticHandler
import PyDS.LinkRewritingHandler
import PyDS.XMLRPCHandler
import PyDS.SOAPHandler

# import API modules
import PyDS.BloggerAPI
import PyDS.MetaWeblogAPI

# import statically installed tools
import PyDS.RenderTool
import PyDS.MacrosTool
import PyDS.PreferencesTool
import PyDS.UpstreamTool
import PyDS.DownstreamTool
import PyDS.CalendarTool
import PyDS.MirrorTool
import PyDS.ThemesTool
import PyDS.WeblogTool
import PyDS.WikiTool
import PyDS.StoryTool
import PyDS.ZineTool
import PyDS.ExternalStoryTool
import PyDS.NavigationTool
import PyDS.DocumentationTool
import PyDS.EventsTool
import PyDS.NuggetsTool
import PyDS.ShortcutTool
import PyDS.StatusTool
import PyDS.TopicExchangeTool
import PyDS.TrackbackTool
import PyDS.PingTool
import PyDS.TimerTool
import PyDS.AggregatorTool
import PyDS.UserTool
import PyDS.PasswordTool
import PyDS.PictureTool
import PyDS.AccessRestrictionsTool
import PyDS.MeshTool
import PyDS.EnclosureTool
import PyDS.BlogmarkTool
import PyDS.ArchiveTool
import PyDS.ShutdownTool

PyDS.timeoutsocket.setDefaultSocketTimeout(_PyDS.sockettimeout)

dtsport = _PyDS.serverport

# just a standard collector for POST data. This one is special in that
# it copies the dynamic context along to the later continue_request call,
# so authentication and remote/local context stuff is available as of the
# call to handle_request
class PyDSCollector:

	def __init__(self, handler, length, request):
		self.handler = handler
		self.request = request
		self.request.collector = self
		self.request.channel.set_terminator(length)
		self.buffer = PyDS.Tool.StringIO()
		self.context = _flet.getContext()

	def collect_incoming_data(self, data):
		self.buffer.write(data)

	def found_terminator(self):
		self.buffer.seek(0)
		self.request.collector = None
		self.request.channel.set_terminator('\r\n\r\n')
		try:
			apply(_flet.begin, [], self.context)
			self.handler.continue_request(self.request, self.buffer)
		finally: _flet.end()


# this class is the handler for PyDS tools. It uses the
# ToolNamespace to decide wether there is a match
class PyDSHandler:

	def __init__(self):
		pass
	
	def match(self, request):
		if PyDS.Tool.mainstatus.isRunning():
			return PyDS.Tool.getToolForRequest(request) is not None
		else:
			return 0

	def handle_request(self, request):
		if request.command.lower() in ('put', 'post'):
			cl = request.get_header('content-length')
			length = int(cl)
			if not cl:
				request.error(411)
			else:
				PyDSCollector(self, length, request)
		else:
			self.continue_request(request, PyDS.Tool.StringIO())

	def continue_request(self, request, input_data):
		PyDS.Tool.dispatcher.append(request, input_data)

AUTHORIZATION = re.compile('Authorization: ([^ ]+) (.*)', re.IGNORECASE)

class AuthHandler:
	
	def __init__(self, handler, usertool, realm='PyDS'):
		self.handler = handler
		self.user = usertool
		self.realm = realm
	
	def match(self, request):
		return self.handler.match(request)
	
	def get_matching_header(self, reg, headers):
		for line in headers:
			m = reg.match(line)
			if m and m.end() == len(line):
				return m.groups()
	
	def handle_request(self, request):
		tool = PyDS.Tool.getToolForRequest(request)
		if not(getattr(tool, 'useAuthentication', 0)):
			try:
				_flet.begin(remote_access=2)
				return self.handler.handle_request(request)
			finally: _flet.end()
		m = self.get_matching_header(AUTHORIZATION, request.header)
		if m:
			needadmin = 0
			if request.command.lower() == 'post':
				needadmin = 1
			path, params, query, fragment = request.split_uri()
			if path.find('_redir') >= 0:
				needadmin = 1
			if not(tool.needAdmin):
				needadmin = 0
			scheme, challenge = m
			scheme = string.lower(scheme)
			if scheme == 'basic':
				cookie = challenge
				try:
					decoded = base64.decodestring(cookie)
				except:
					request.error(400)
					return
				user, password = string.split(decoded, ':')
				if self.user.checkUser(user, password, needadmin):
					request.authenticated_user = user
					try:
						_flet.begin(authenticated_user=user, remote_access=1)
						return self.handler.handle_request(request)
					finally: _flet.end()
				else:
					return self.handle_unauthorized(request)
			else:
				request.error(400)
				return
		else:
			return self.handle_unauthorized(request)
		
	def handle_unauthorized(self, request):
		request.channel.set_terminator(None)
		request['Connection'] = 'close'
		request['WWW-Authenticate'] = 'Basic realm="%s"' % self.realm
		request.error(401)
		return
		
def terminate(signal="(API shutdown)", param=None):
	"""Signal handler for the pycs daemon.  Applicable
	only to those systems implementing POSIX signals.
	"""

	PyDS.Tool.log(_('The server is shutting down'))
	# first phase shutdown, only stop background threads and gather
	# the next shutdown stage to run
	tools = PyDS.Tool.getToolsForShutdown()
	for tool in tools:
		if hasattr(tool, '_shutdown'):
			try: tool._shutdown()
			except: pass # can't do much if exception occurs - maybe print report
	PyDS.Tool.mainstatus.running(0)
	# second phase shutdown, stop the background threads and
	# call the remove hook
	PyDS.Tool.dispatcher.stop()
	for tool in tools:
		if hasattr(tool, '_remove'):
			try: tool._remove()
			except: pass # can't do much if exception occurs - maybe print report
	ctime = time.asctime( time.localtime( time.time() ) )
	print _("Received signal %s terminating at %s") % (signal, ctime)
	os.remove( _PyDS.PIDFILE )
	sys.exit( 0 )

def install_handlers():
	import signal
	print _("Installing signal handlers")
	#install default signal profile
	signalHandlers = (
		('SIGTTOU', signal.SIG_IGN),
		('SIGTTIN', signal.SIG_IGN),
		('SIGCHLD', signal.SIG_IGN),
		('SIGTSTP', terminate),
		('SIGTERM', terminate),
		('SIGINT', terminate),
	)
	# register the handler that are supported on this platform
	for (sigID, function) in signalHandlers:
		sigNumber = getattr(signal, sigID, None)
		if sigNumber != None:
			signal.signal(sigNumber, function)

def getCacheMimeType(path):
	downstream = PyDS.Tool.getTool('downstream')
	return downstream.getCacheMimeType(path)

def start_server(daemon=1, restore=None):
	install_handlers()
	if daemon:

		for file in ('etc.log', 'error.log'):
			fname = os.path.join(_PyDS.LOGDIR, file)
			if os.path.exists(fname):
				os.unlink(fname)

		print _('Changing log redirection')
		become_daemon(
			_PyDS.ROOTDIR,
			os.path.join(_PyDS.LOGDIR, 'etc.log'),
			os.path.join(_PyDS.LOGDIR, 'error.log')
		)
	else:
		# if not daemon, still change current directory (otherwise
		# that's a side effect of become_daemon)
		os.chdir(_PyDS.ROOTDIR)

	# import dynamically installed tools in the tools directory
	tlist = glob.glob("%s/*.py" % _PyDS.MASTERTOOLDIR)
	tlist.extend(glob.glob('%s/*.py' % _PyDS.TOOLDIR))
	sys.path.insert(0, _PyDS.MASTERTOOLDIR)
	sys.path.insert(0, _PyDS.TOOLDIR)
	for tool in tlist:
		print _("Installing tool %s") % tool
		eval(compile("import %s" % os.path.splitext(os.path.basename(tool))[0], tool, 'exec'))

	hs = None
	accessLog = None
	log = None

	ms = None

	PyDS.Tool.mainstatus.running(1)
	PyDS.Tool.initTools(PyDS.Tool.mainstatus)
	PyDS.Tool.initProcessor()

	if not restore:
		# install output trigger
		_PyDS.select_trigger = PyDS.select_trigger.trigger()

		accessLog = logger.rotating_file_logger(
			os.path.join(_PyDS.LOGDIR, 'access.log'), None, 1024*1024
		)
		log = status_handler.logger_for_status(accessLog)

		hs = http_server.http_server(
			_PyDS.serverip,
			dtsport,
			None,
			log
		)
		hs.server_name = _PyDS.serverhostname

		if _PyDS.monitorport and _PyDS.monitorpassword:
			ms = monitor.secure_monitor_server(
				_PyDS.monitorpassword,
				_PyDS.serverip,
				_PyDS.monitorport
			)

	mypid = os.getpid()
	pidf = open(_PyDS.PIDFILE, "w")
	pidf.write('%d\n%d' % (mypid, dtsport))
	pidf.close()

	if not(restore):
		# desktop server
		th = PyDSHandler()
		hs.install_handler(th)

		fs1 = filesys.os_filesystem(_PyDS.STATICDIR)
		fh1 = PyDS.StaticHandler.static_handler('/static', fs1)
		hs.install_handler(fh1)
	
		fs2 = filesys.os_filesystem(_PyDS.IMAGESDIR)
		fh2 = PyDS.StaticHandler.static_handler('/images', fs2)
		hs.install_handler(fh2)

		fs3 = filesys.os_filesystem(os.path.join(_PyDS.VARDIR, 'cache'))
		fh3 = PyDS.StaticHandler.static_handler('/cache', fs3, getCacheMimeType)
		hs.install_handler(fh3)

		pfs = filesys.os_filesystem(_PyDS.ROOTDIR)
		pfh = PyDS.LinkRewritingHandler.link_rewriting_handler('/preview', pfs)
		hs.install_handler(pfh)

		xmlh = PyDS.XMLRPCHandler.xmlrpc_handler()
		hs.install_handler(xmlh)
	
		soaph = PyDS.SOAPHandler.soap_handler()
		hs.install_handler(soaph)
	
		if _PyDS.remotehostname and _PyDS.remoteip:
			rs = http_server.http_server(
				_PyDS.remoteip,
				_PyDS.remoteport,
				None,
				log
			)
			rs.server_name = _PyDS.remotehostname
			user = PyDS.Tool.getTool('user')
			rs.install_handler(AuthHandler(th,user))
			rs.install_handler(xmlh)
			rs.install_handler(soaph)
			rs.install_handler(fh1)
			rs.install_handler(fh2)
			rs.install_handler(fh3)
			rs.install_handler(AuthHandler(pfh,user))
			
	if restore:
		_PyDS.echoLog = 1
		PyDS.Tool.log(_('Starting the restore process'))
		PyDS.Tool.getTool('upstream')._restore(restore)
	else:
		print _("Server is now up and responding")
		PyDS.Tool.log(_('The server has started'))
		try:
			# set up default dynamic name bindings
			_flet.begin(
				authenticated_user='',
				remote_access=0,
			)
			asyncore.loop()
		finally:
			_flet.end()


# copy stuff over from share/pyds to the www subdirectories if missing
liste_static = []
liste_images = []
for fn in os.listdir(_PyDS.SHAREDDIR):
	if fn[-4:] in ('.css',):
		liste_static.append(fn)
	elif fn[-4:] in ('.gif', '.png', '.jpg'):
		liste_images.append(fn)
for fn in  liste_static:
	if not(os.path.exists(os.path.join(_PyDS.STATICDIR, fn))):
		shutil.copy(os.path.join(_PyDS.SHAREDDIR, fn),
			os.path.join(_PyDS.STATICDIR, fn))
for fn in  liste_images:
	if not(os.path.exists(os.path.join(_PyDS.IMAGESDIR, fn))):
		shutil.copy(os.path.join(_PyDS.SHAREDDIR, fn),
			os.path.join(_PyDS.IMAGESDIR, fn))

