"""

Python Desktop Server - Themes Tool

Copyright (c) 2002, Georg Bauer <gb@murphy.bofh.ms>

Permission is hereby granted, free of charge, to any person obtaining a copy of 
this software and associated documentation files (the "Software"), to deal in 
the Software without restriction, including without limitation the rights to 
use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies of
 
the Software, and to permit persons to whom the Software is furnished to do so, 
subject to the following conditions:

The above copyright notice and this permission notice shall be included in all 
copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR 
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS
 
FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR 
COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER 
IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN 
CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.

"""

# $Id: ThemesTool.py,v 1.15 2004/01/28 14:32:14 gb Exp $

import os
import re
import time
import glob
import shutil

from zipfile import ZipFile, ZipInfo

import PyDS.Tool

class ThemesTool(PyDS.Tool.StandardTool):

	def _initdb(self):
		self.templates = {
			'pyds.css': (
				_('stylesheet for desktop and cloud'),
				'www/static/pyds.css'
			),
			'BaseTemplate': (
				_('basic template for desktop website'),
				'templates/BaseTemplate.tmpl'
			),
			'BaseRendering': (
				_('basic template for cloud'),
				'templates/BaseRendering.tmpl'
			)
		}
		self.importantTool = 1
	
	def _initopts(self):
		self.themesdir = os.path.join(_PyDS.VARDIR, 'themes')
		if not(os.path.isdir(self.themesdir)):
			os.makedirs(self.themesdir)
	
	def _rerender(self):
		publicdir = os.path.join(_PyDS.ROOTDIR, self.name)
		if os.path.isdir(publicdir):
			html = "<ul>"
			themes = glob.glob('%s/*.zip' % publicdir)
			for theme in themes:
				bname = os.path.basename(theme)
				name = os.path.splitext(bname)[0]
				desc = ZipFile(theme).read('theme.txt')
				html += '<li><b><a href="%s">%s</a>:</b> %s</li>' % (
					self.getStaticUrl(bname),
					name,
					desc
				)
			html += "</ul>"
			req = self._getDummyRequest()
			req.setLocalValue('title', _("All published Themes"))
			body = req.renderPage('BaseRendering',category='',body=html)
			self.storeHTML(body)
	
	# this method hooks a template name into the themes tool
	def hookTemplate(self, name, desc, path):
		self.templates[name] = (desc, path)
		return name
	
	# this method lists templates that are hooked into the ThemesTool
	def listTemplates(self):
		return self.templates.keys()
	
	# this method returns the details of a hooked template
	def getTemplateDetails(self, name):
		return self.templates[name]
	
	# return the content of a named template
	def getTemplate(self, name):
		path = self.templates[name][1]
		match = re.match(r'templates/(.*)\.tmpl$', path)
		if match:
			PyDS.Tool.getTemplate(match.group(1))
		path = os.path.join(_PyDS.HOMEDIR, self.templates[name][1])
		path = os.path.normpath(path)
		if len(path) < len(_PyDS.HOMEDIR):
			return ''
		else:
			if path[:len(_PyDS.HOMEDIR)] != _PyDS.HOMEDIR:
				return ''
		f = open(path)
		str = f.read()
		f.close()
		return str
	
	# change the content of a named template in the filesystem
	def putTemplate(self, name, content):
		path = os.path.join(_PyDS.HOMEDIR, self.templates[name][1])
		path = os.path.normpath(path)
		if len(path) < len(_PyDS.HOMEDIR):
			return ''
		else:
			if path[:len(_PyDS.HOMEDIR)] != _PyDS.HOMEDIR:
				return ''
		f = open(path, 'w')
		f.write(content)
		f.close()
		return name
	
	# write an image to the filesystem
	def putImage(self, name, content):
		path = os.path.join(_PyDS.ROOTDIR, 'images', self.templates[name][1])
		path = os.path.normpath(path)
		if len(path) < len(_PyDS.ROOTDIR):
			return ''
		else:
			if path[:len(_PyDS.ROOTDIR)] != _PyDS.ROOTDIR:
				return ''
		f = open(path, 'w')
		f.write(content)
		f.close()
		return name
	
	# get a list of files in the images subfolder that are referenced by the templates
	def getReferencedImages(self):
		res = []
		src = ''
		for template in self.templates.keys():
			src += self.getTemplate(template)
		defaultImages = {}
		for name in os.listdir(_PyDS.SHAREDDIR):
			defaultImages[name] = 1
		for name in os.listdir(os.path.join(_PyDS.ROOTDIR, 'images')):
			fullname = os.path.join(_PyDS.ROOTDIR, 'images', name)
			if not(defaultImages.has_key(name)) and os.path.isfile(fullname) and src.find(name) >= 0:
				res.append(name)
		return res

	def index_html(self, req):
		macros = self.getToolByNamespace('macros')
		html = '<p><a href="%s">%s</a>' % (
			self.getUrl('themes_html'),
			_('Click here to load a predefined theme')
		)
		html += "<ul>"
		tkeys = self.templates.keys()
		tkeys.sort()
		for template in tkeys:
			html += '<li><b><a href="%s">%s</a>:</b> %s</li>' % (
				self.getUrl('show_html', id=template),
				template,
				self.templates[template][0],
			)
		for image in self.getReferencedImages():
			html += '<li>%s</li>' % macros.imageTag('/images/%s' % image)
		html += "</ul>"
		html += _('<h2>Create a new theme from your current templates below</h2>')
		html += '<p><form action="%s" method="POST">' % (
			self.getUrl('create_redir')
		)
		html += '<table border=0>'
		html += '<tr>'
		html += '<th valign="top" align="right">%s</th><td><input name="name" size=30></td>' % (
			_('Name')
		)
		html += '</tr>'
		html += '<tr>'
		html += '<th>%s</th><td>' % _('publish')
		html += '<select name="publish">'
		html += '<option value="0">%s' % _('no')
		html += '<option value="1">%s' % _('yes')
		html += '</select>'
		html += '</td>'
		html += '</tr>'
		html += '<tr>'
		html += '<th valign="top" align="right">%s</th><td><textarea name="desc" rows=5 cols=30></textarea></td>' % (
			_('Description')
		)
		html += '</tr>'
		html += '<tr><td colspan=2 align="right">'
		html += '<input type=submit name="ThemeCreate" value="%s">' % _('save')
		html += '</td></tr>'
		html += '</table>'
		html += '</form>'
		req.setLocalValue('title', _("Your Templates"))
		return req.renderPage('BaseTemplate', body=html)
		
	def themes_html(self, req):
		macros = self.getToolByNamespace('macros')
		html = _('<p>As soon as you click on the name of a theme, this theme will be loaded into your PyDS instance and overwrite all current template contents!')
		html += "<ul>"
		themes = glob.glob('%s/*.zip' % self.themesdir)
		for theme in themes:
			name = os.path.splitext(os.path.basename(theme))[0]
			desc = ZipFile(theme).read('theme.txt')
			html += '<li><b><a href="%s">%s</a>:</b> %s</li>' % (
				self.getUrl('load_redir', name=name),
				name,
				desc
			)
		html += "</ul>"
		html += '<p align="right"><a href="%s">%s</a>' % (
			self.getUrl(),
			_('back to the template overview')
		)
		req.setLocalValue('title', _("Your predefined Themes"))
		return req.renderPage('BaseTemplate', body=html)

	def show_html(self, req):
		macros = self.getToolByNamespace('macros')
		id = req.getFirstValue('id')
		html = '<p><a href="%s">%s</a></p>' % (
			self.getUrl('edit_html', id=str(id)),
			_('Click here to change this template')
		)
		html += '<pre class="whiteboxsmall">%s</pre>' % (
			macros.quoteHTML(self.getTemplate(id))
		)
		html += '<p align="right"><a href="%s">%s</a>' % (
			self.getUrl(),
			_('choose a different template')
		)
		req.setLocalValue('title', id)
		return req.renderPage('BaseTemplate', body=html, withcalendar=0)
		
	def edit_html(self, req):
		macros = self.getToolByNamespace('macros')
		id = req.getFirstValue('id')
		template = self.getTemplate(id)
		html = '<form action="%s" method="POST">' % self.getUrl('edit_redir')
		html += '<input type="hidden" name="id" value="%s">' % id
		html += '<table border=0>'
		html += '<tr>'
		html += '<th valign="top" align="right">%s</th><td><textarea nowrap name="source" rows=30 cols=60>%s</textarea></td>' % (
			_('Source'), macros.quoteHTML(template)
		)
		html += '</tr>'
		html += '<tr><td colspan=2 align="right">'
		html += '<input type=submit name="TemplateSave" value="%s">' % _('save')
		html += '</td></tr>'
		html += '</table>'
		html += '</form>'
		html += '<p align="right"><a href="%s">%s</a>' % (
			self.getUrl(),
			_('choose a different template')
		)
		req.setLocalValue('title', id)
		return req.renderPage('BaseTemplate', body=html, withcalendar=0)
		
	def edit_redir(self, req):
		id = req.getFirstValue('id')
		source = req.getFirstValue('source')
		self.putTemplate(id, source)
		return self.getUrl('show_html', id=id)
	
	def create_redir(self, req):
		upstream = self.getToolByNamespace('upstream')
		name = req.getFirstValue('name')
		name = re.sub('\s+', '_', name)
		desc = req.getFirstValue('desc')
		publish = int(req.getFirstValue('publish'))
		if name:
			path = os.path.join(self.themesdir, '%s.zip' % name)
			zf = ZipFile(path, "w")
			for template in self.templates.keys():
				self.getTemplate(template)
				tpath = os.path.join(_PyDS.HOMEDIR, self.templates[template][1])
				zf.write(tpath, template)
			imagelist = ''
			for name in self.getReferencedImages():
				zf.write(os.path.join(_PyDS.ROOTDIR, 'images', name), name)
				imagelist += name + '\n'
			zf.writestr(ZipInfo('theme.txt'), desc)
			zf.writestr(ZipInfo('images.txt'), imagelist)
			zf.close()
			publicdir = os.path.join(_PyDS.ROOTDIR, self.name)
			publicname = os.path.join(publicdir, '%s.zip' % name)
			if publish:
				if not(os.path.isdir(publicdir)):
					os.makedirs(publicdir)
				shutil.copyfile(path, publicname)
			else:
				if os.path.exists(publicname):
					os.unlink(publicname)
			self._rerender()
		return self.getUrl('index_html')
	
	def load_redir(self, req):
		name = req.getFirstValue('name')
		if name:
			path = os.path.join(self.themesdir, '%s.zip' % name)
			zf = ZipFile(path)
			hasimages = 0
			for name in zf.namelist():
				if name == 'images.txt':
					hasimages = 1
				elif name != 'theme.txt':
					str = zf.read(name)
					# normalize line endings, since we don't
					# know who wrote the zipfile and what
					# that platforms line endings are
					str = str.replace('\x0d\x0a', '\n')
					str = str.replace('\r','\n')
					self.putTemplate(name, str)
			if hasimages:
				for image in zf.read('images.txt').split('\n'):
					if image:
						img = zf.read(image)
						self.putImage(image, img)
		return self.getUrl('index_html')


__desc__ = _("This is a tool that manages your templates.")

PyDS.Tool.registerTool(ThemesTool, 'themes', _('Themes'), __desc__, 850)

