"""

Python Desktop Server - Trackback client

Copyright (c) 2003, Georg Bauer <gb@murphy.bofh.ms>

Permission is hereby granted, free of charge, to any person obtaining a copy of 
this software and associated documentation files (the "Software"), to deal in 
the Software without restriction, including without limitation the rights to 
use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies of
 
the Software, and to permit persons to whom the Software is furnished to do so, 
subject to the following conditions:

The above copyright notice and this permission notice shall be included in all 
copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR 
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS
 
FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR 
COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER 
IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN 
CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.

"""

# $Id: TrackbackTool.py,v 1.12 2004/02/04 09:56:39 gb Exp $

import re
import time
import urllib

import PyDS.Tool

from PyDS.PersistentQueue import PersistentQueue

class TrackbackTool(PyDS.Tool.StandardTool):

	def _createdb(klass, db):
		db.getas("trackbackurls[url:S,name:S]")

	_createdb = classmethod(_createdb)

	def _initdb(self):
		try:
			self._acquire()
			self.trackbackurls = self.db.view('trackbackurls').ordered(1)
			self.queue = PersistentQueue(self.name, 'trackback')
		finally: self._release()
		
	def _initactive(self):
		weblog = self.getToolByNamespace('weblog')
		weblog._addHook('edit', 'select', _('Trackback'), self._trackbackPingForm, self._trackbackActive)
		weblog._addHook('editold', 'ping', '', self._trackbackPingHook, self._trackbackActive)
		weblog._addHook('postnew', 'ping', '', self._trackbackPingHook, self._trackbackActive)
	
	def _backup(self):
		return self._backupTable('trackbackurls')

	def _initthread(self, server):
		pref = self.getPreferencesTool()
		while server.isRunning():
			time.sleep(10)
			if self.queue.length() and pref.online():
				while self.queue.length():
					(cmd, key, data) = self.queue.shift()
					jobf = getattr(self, cmd)
					jobf(key, data)
				self.logVerbose(_('work done, going to sleep again'))

	def _trackbackActive(self, *args, **kw):
		return 1
	
	def _trackbackPingForm(self, req, *args, **kw):
		macros = self.getToolByNamespace('macros')
		html = '<textarea name="tburl" cols=40 rows=3>%s</textarea>' % macros.quoteHTML('\n'.join(req.getAllValues('ptburl')))
		liste = self.getTrackbackTargets()
		if len(liste):
			html += '<br>'
			for row in liste:
				html += '<span class="checkbox"><input type=checkbox name="tbaddurl" value="%s">&nbsp;%s</span> ' % (row['url'], row['name'])
		return html

	def _trackbackPingHook(self, tool, req, id, *args, **kw):
		tburls = req.getFirstValue('tburl')
		if tburls:
		    for tburl in tburls.split('\n'):
		    	if tburl:
				self.queue.append(
					'_trackbackPing',
					(tburl, tool.name, id),
					None
				)
		tburls = req.getAllValues('tbaddurl')
		for tburl in tburls:
			self.queue.append(
				'_trackbackPing',
				(tburl, tool.name, id),
				None
			)
	
	def _trackbackPing(self, key, data):
		(url, toolname, id) = key
		tool = self.getToolByNamespace(toolname)
		res = self.trackbackPing(url, tool, id)
		if res:
			tool.log(_('Trackback error: %s'), res)
	
	def getPreferencePanels(self):
		return [
			['trackback_panel', _('Trackbacks'),
				_('Here you can predefine additional trackback URLs for postings.')]
		]
			

	# This method returns a list of dictionaries with configured additional trackback pings
	def getTrackbackTargets(self):
		try:
			self._acquire()
			liste = []
			for row in self.trackbackurls:
				liste.append({
					'url':row.url,
					'name':row.name
				})
			return liste
		finally:
			self._release()

	def trackback_panel(self, req):
		macros = self.getToolByNamespace('macros')
		html = _('<p>Your configured additional trackback URLs')
		html += '<form action="%s" method="POST">' % self.getUrl('trackback_redir')
		html += '<table border=0 width="100%">'
		html += '<tr>'
		html += '<th>%s</th><th align="left">%s</th><th>%s</th>' % (_('Delete'), _('Name'), _('URL'))
		html += '</tr>'
		for tb in self.getTrackbackTargets():
			html += '<tr>'
			html += '<td><input type=checkbox name="delete" value="%s"></td>' % tb['url']
			html += '<td>%s</td>' % tb['name']
			html += '<td>%s</td>' % tb['url']
			html += '</tr>'
		html += '<tr>'
		html += '<th>%s</th>' % _('Add')
		html += '<td><input name="tbname" size=10></td>'
		html += '<td><input name="tburl" size=30></td>'
		html += '</tr>'
		html += '</table>'
		html += '<input type=submit name="TrackbackSubmit" value="%s">' % _('do it')
		html += '</form>'
		return html
	
	def trackback_redir(self, req):
		try:
			self._acquire()
			for tburl in req.getAllValues('delete'):
				(idx, found) = self.trackbackurls.locate({'url': tburl})
				if found:
					self.trackbackurls.delete(idx)
			tburl = req.getFirstValue('tburl')
			tbname = req.getFirstValue('tbname')
			if tburl and tbname:
				self.trackbackurls.append({
					'url': tburl,
					'name': tbname
				})
			self._commit()
			return req.getUrl(
				'panel_html',
				tool='preferences',
				panel='trackback_panel',
				fortool='trackback'
			)
		finally: self._release()

	# this method pings a given URL with the trackback protocol
	def trackbackPing(self, url, tool, id):

		def stripHTML(str):
			macros = self.getToolByNamespace('macros')
			str = macros.unquoteXML(str)
			return macros.stripHTML(str)

		item = tool.getRSSItem(id, fullrss=1)
		excerpt = stripHTML(item['description'])
		excerpt = re.sub('\s+', ' ', excerpt)
		if len(excerpt) > 250:
			excerpt = excerpt[:250]
			while len(excerpt) > 200 and excerpt[-1] != ' ':
				excerpt = excerpt[:-1]
			excerpt += ' ...'
		data = urllib.urlencode({
			'blog_name': stripHTML(tool.getRSSTitle()),
			'title': stripHTML(item['title']),
			'url': item['guid'],
			'excerpt': excerpt
		})
		downstream = self.getToolByNamespace('downstream')
		opener = downstream.getUrlOpener(1)
		xml = opener.open(url, data).read()
		if xml[:5] == '<?xml':
			m1 = re.match(r'.*<response>.*<error>(.*?)</error>.*</response>.*', xml, re.DOTALL)
			if m1:
				if m1.group(1).strip() == '0':
					return ''
				else:
					m2 = re.match(r'.*<response>.*<message>(.*?)</message>.*</response>.*', xml, re.DOTALL)
					if m2:
						return m2.group(1)
					else:
						return _('unspecified error in trackback ping')
			else:
				return _('No trackback response found')
		else:
			return _('No XML format found in trackback response')
	
	def _status(self):
		try:
			self._acquire()
			html = PyDS.Tool.StandardTool._status(self)
			html += '<ul>'
			html += _('<li>There are %d pings in the queue</li>') % self.queue.length()
			html += self._threadstatus()
			html += '</ul>'
			return html
		finally: self._release()
	
__desc__ = _("This tool implements trackback client suppport.")

PyDS.Tool.registerTool(TrackbackTool, 'trackback', _('Trackback Ping'), __desc__, 1310)

