"""

Python Desktop Server - Upstreaming Tool

Copyright (c) 2002, Georg Bauer <gb@murphy.bofh.ms>

Permission is hereby granted, free of charge, to any person obtaining a copy of 
this software and associated documentation files (the "Software"), to deal in 
the Software without restriction, including without limitation the rights to 
use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies of
 
the Software, and to permit persons to whom the Software is furnished to do so, 
subject to the following conditions:

The above copyright notice and this permission notice shall be included in all 
copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR 
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS
 
FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR 
COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER 
IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN 
CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.

"""

# $Id: UpstreamTool.py,v 1.103 2004/03/17 11:01:35 gb Exp $

import os
import re
import sys
import md5
import time
import stat
import string
import metakit
import xmlrpclib
import threading
import urllib

import PyDS.Tool
import SOAPpy

import XSSDriver
import FTPDriver
import LFSDriver

from DriverRegistry import getDriver

rssfeedtitlere = re.compile(r'<title>(.*?)</title>')
def getRSSFeedTitle(xml):
	match = rssfeedtitlere.search(xml)
	if match:
		return match.group(1)
	else:
		return PyDS.Tools.getTool('preferences').get('private','title')

rssfeedlinkre = re.compile(r'<link>(.*?)</link>')
def getRSSFeedLink(xml):
	match = rssfeedlinkre.search(xml)
	if match:
		return match.group(1)
	else:
		return None

class Unmarshaller(xmlrpclib.Unmarshaller):

	dispatch = xmlrpclib.Unmarshaller.dispatch

	def set_resultencoding(self, encoding):
		self.resultencoding = encoding

	def end_string(self, data):
		if self._encoding:
			data = xmlrpclib._decode(data, self._encoding)
		str = xmlrpclib._stringify(data)
		if type(str) == type(u''):
			if hasattr(self, 'resultencoding'):
				str = str.encode(self.resultencoding)
			else:
				str = str.encode('ISO-8859-1')
		self.append(str)
		self._value = 0
	dispatch["string"] = end_string
	dispatch["name"] = end_string

class PatchingTransport(xmlrpclib.Transport):

	user_agent = PyDS.Tool.version

	def getparser(self):
		u = Unmarshaller()
		u.set_resultencoding(_PyDS.documentEncoding)
		if xmlrpclib.FastParser:
			p = xmlrpclib.FastParser(u)
		elif xmlrpclib.SgmlopParser:
			p = xmlrpclib.SgmlopParser(u)
		elif xmlrpclib.ExpatParser:
			p = xmlrpclib.ExpatParser(u)
		else:
			p = xmlrpclib.SlowParser(u)
		return (p, u)

	def send_user_agent(self, connection):
		connection.putheader('User-Agent', PyDS.Tool.version)

	def parse_response(self, f):
		# read response and try to parse it
		p, u = self.getparser()

		response = f.read()

		try:
			p.feed(response)
		except:
			if _PyDS.verbose:
				print _('retrying XML parsing with latin1 encoding')
			p, u = self.getparser()
			response = response.replace(
				'<?xml version="1.0"?>',
				'<?xml version="1.0" encoding="iso-8859-1"?>'
			)
			p.feed(response)

		f.close()
		p.close()

		return u.close()

class ProxiedTransport(PatchingTransport):

	def set_proxy(self, proxy):
		self.proxy = proxy

	def make_connection(self, host):
		self.realhost = host
		import httplib
		return httplib.HTTP(self.proxy)

	def send_request(self, connection, handler, request_body):
		connection.putrequest("POST", 'http://%s%s' % (self.realhost, handler))

	def send_host(self, connection, host):
		connection.putheader('Host', self.realhost)
	
class UpstreamLinkTool(PyDS.Tool.LinkTool):

	def setParentVariable(self, tool, table, field, withusernum=0):
		self.tool = tool
		self.table = table
		self.field = field
		self.withusernum = withusernum
	
	def nugget(self, url=None):
		if not(url):
			if self.table == 'prefs':
				if hasattr(self.tool.prefs[0], self.field):
					url = getattr(self.tool.prefs[0], self.field)
			elif self.table == 'status':
				if hasattr(self.tool.status[0], self.field):
					url = getattr(self.tool.status[0], self.field)
			else:
				return ''
		if url:
			if self.withusernum:
				url += self.tool.prefs[0].usernum
			return '<a target="_blank" href="%s">%s</a>: %s' % (
				url, self.title, self.desc
			)
		else:
			return ''

class UpstreamTool(PyDS.Tool.StandardTool):

	def _createdb(klass, db):
		db.getas("prefs[rpcurl:S,password:S,pinginterval:I,upstreaminterval:I,doupstream:I,doping:I,usernum:S,proxy:S,commentsurl:S,cloudurl:S,bytesused:I,maxbytes:I,maxfilesize:I,rankingurl:S,refererurl:S,mailtourl:S,updatesurl:S,webbugurl:S,cancomment:I,trackbackurl:S,cantrackback:I,canmirrorposts:I,hassearch:I,urlsearch:S]")
		db.getas("status[email:S,name:S,organization:S,deletes:I,membersince:S,pings:I,serial:S,url:S,behindwall:I,title:S,upstreams:I,signons:I,lastping:S,lastupstream:S,port:I,bytesupstreamed:I,bytesinuse:I,lastsignoff:S,lastsignon:S,signedon:I,usernum:S]")
		db.getas("filestat[path:S,time:I,size:I]")

	_createdb = classmethod(_createdb)

	def _initdb(self):
		try:
			self._acquire()
			self.prefs = self.db.view('prefs')
			if len(self.prefs) == 0:
				self.prefs.append(usernum=_('unregistered'))
			if not(self.prefs[0].pinginterval):
				self.prefs[0].pinginterval = 900
			if not(self.prefs[0].upstreaminterval):
				self.prefs[0].upstreaminterval = 900
			self.status = self.db.view('status')
			self.filestatus = self.db.view('filestat').ordered(1)
			if len(self.status) == 0:
				self.status.append(usernum=self.prefs[0].usernum)
			self._commit()
			self.hooks = {}
			self.importantTool = 1
		finally: self._release()
	
	def _restoreTable(self, table, items, purge=0):
		try:
			self._acquire()
			password = self.prefs[0].password
			proxy = self.get('prefs', 'proxy')
			PyDS.Tool.StandardTool._restoreTable(self, table, items, purge)
			self.prefs[0].password = password
			self.prefs[0].proxy = proxy
			self._commit()
		finally: self._release()

	def _backup(self):
		l = self._backupTable('prefs', exceptcols=['password','proxy'])
		l.extend(self._backupTable('status'))
		l.extend(self._backupTable('filestatus'))
		l.extend(self._driver._backup())
		return l

	def _initopts(self):
		self._awakeUpstream = 0
		self.showCalendar = 0
		prefs = self.getPreferencesTool()
		driver = prefs.get('private', 'driver')
		self._driver = getDriver(driver or 'XSS', self)
		self._driver.initdb()
		self._driver.initopts()
	
	def _initactive(self):
		timer = self.getToolByNamespace('timer')
		timer._addHook('select', 'upstream', _('backup databases'), self._backupTimerHook)
		timer._addHook('select', 'upstream2', _('rerender content of tool'), self._rerenderTimerHook)

	def _initsubtools(self):
		tools = []

		t = UpstreamLinkTool('cloud', _('Cloud'), _('This link goes to your cloud page.'), level=810)
		t.setParentVariable(self, 'prefs', 'cloudurl')
		tools.append(t)

		t = PyDS.Tool.LinkTool('preview', _('Preview'), _('This link gives you a preview of how your cloud will look like.'), url='%spreview/' % _PyDS.pathprefix, level=815, target='_blank')
		tools.append(t)

		t = UpstreamLinkTool('referers', _('Referrers'), _('This link goes to your clouds referrers page.'), level=820)
		t.setParentVariable(self, 'prefs', 'refererurl', 1)
		tools.append(t)

		t = UpstreamLinkTool('rankings', _('Rankings'), _('This link goes to your clouds rankings page.'), level=822)
		t.setParentVariable(self, 'prefs', 'rankingurl')
		tools.append(t)

		t = UpstreamLinkTool('updates', _('Updates'), _('This link goes to your clouds weblog updates page.'), level=824)
		t.setParentVariable(self, 'prefs', 'updatesurl')
		tools.append(t)

		return tools

	def _initthread(self, server):
		pref = self.getPreferencesTool()
		now = time.time()
		nextPing = now + self.get('prefs', 'pinginterval')
		nextUpstream = now + self.get('prefs', 'upstreaminterval')
		firstPing = 1
		while server.isRunning():
			time.sleep(10)
			now += 10
			if now > nextPing:
				if self.needsStuff():
					self.log(_('Config missing, skipping ping'))
				elif pref.get('private', 'offline'):
					self.logVerbose(_('System offline, skipping ping'))
				elif not(self.get('prefs','doping')) and not(firstPing):
					self.logVerbose(_('Ping disabled, skipping ping'))
				else:
					self.pingCloud(online=1)
					firstPing = 0
				nextPing = now + self.get('prefs','pinginterval')
			if (now > nextUpstream) or self._awakeUpstream:
				awake = self._awakeUpstream
				self._awakeUpstream = 0
				if self.needsStuff():
					self.log(_('Config missing, skipping upstream'))
				elif pref.offline():
					self.logVerbose(_('System offline, skipping upstream'))
				elif not(self.get('prefs','doupstream')):
					self.logVerbose(_('Upstream disabled, skipping upstream'))
				else:
					maxsize = self.get('prefs','maxfilesize')
					if not(maxsize):
						maxsize = 100000
					(tosave, todel) = self.getChangeSet()
					if len(tosave):
						dosave = []
						savesize = 0
						for frec in tosave:
							if savesize < maxsize:
								dosave.append(frec[0])
								savesize += frec[1]
							else:
								self._awakeUpstream = 1
						if len(dosave) > 25:
							dosave = dosave[:25]
							self._awakeUpstream = 1
						self.log(_('about to save: %s'), dosave)
						self.saveMultipleFiles(dosave)
					if len(todel):
						todel = map(lambda r: r[0], todel)
						if len(todel) > 25:
							todel = todel[:25]
							self._awakeUpstream = 1
						self.log(_('about to delete: %s'), todel)
						self.deleteMultipleFiles(todel)
					if len(tosave) or len(todel) and not(self._awakeUpstream):
						self.logVerbose(_('work done, going to sleep again'))
					elif awake:
						self.logVerbose(_('nothing to do, going to sleep again'))
				nextUpstream = now + self.get('prefs','upstreaminterval')
		
	def _shutdown(self):
		self.log(_('signoff from cloud'))
		self.pingCloud(online=0)

	def _status(self):
		try:
			self._acquire()
			html = PyDS.Tool.StandardTool._status(self)
			html += '<ul>'
			html += self._threadstatus()
			html += '</ul>'
			return html
		finally: self._release()

	def _restore(self, url):
		downstream = self.getToolByNamespace('downstream')
		remote = 0
		if len(url) > 7 and url[:7] == 'http://':
			self.log(_('reading remote backup from %s'), url)
			f = downstream.download(url, force=1)
			remote = 1
		else:
			self.log(_('reading local backup from %s'), url)
			f = open(url)
		prefix = url[:url.rfind('backup')]
		filere = re.compile(r'\s+<file\s+path="/?(backup/.*?)".*/>')
		seen = {}
		for line in f.readlines():
			m = filere.match(line)
			if m:
				fn = m.group(1)
				self.logVerbose(_('matched file %s') % fn)
				if remote:
					fn = prefix + fn
				else:
					fn = prefix + fn.replace('/', os.sep)
				self.logVerbose(_('URL is %s') % fn)
				(toolname, tablename, items) = self.parseBackup(fn)
				tool = self.getToolByNamespace(toolname)
				if tool:
					table = tool._restoreGetTable(tablename)
					if table != None:
						purge = 0
						if seen.has_key(toolname):
							if not(seen[toolname].has_key(tablename)):
								purge = 1
								seen[toolname][tablename] = 1
						else:
							purge = 1
							seen[toolname] = {tablename:1}
						if purge:
							self.log(_('restoring table %s in tool %s with purge'), tablename, toolname)
						else:
							self.log(_('restoring table %s in tool %s'), tablename, toolname)
						tool._restoreTable(table, items, purge=purge)

	def _backupTimerHook(self, qual, item=None):
		if qual == 'title':
			return _('backup databases')
		elif qual == 'select':
			return '<input type="hidden" name="item" value="all">'
		elif qual == 'item':
			return _('backup databases')
		elif qual == 'run':
			self.backup()
		else:
			raise ValueError('%s is an unknown qualifier' % qual)

	def _rerenderTimerHook(self, qual, item=None):
		if qual == 'title':
			return _('rerender content of tool')
		elif qual == 'select':
			html = '<p><select name="item">'
			try:
				self._acquire()
				liste = []
				for tool in PyDS.Tool.getTools():
					if hasattr(tool, '_rerender'):
						liste.append(tool.name)
				liste.sort()
				for toolname in liste:
					html += '<option value="%s">%s' % (
						toolname,
						toolname
					)
				html += '</select>'
			finally: self._release()
			return html
		elif qual == 'item':
			return _('rebuilding content for tool "%s"') % item
		elif qual == 'run':
			tool = self.getToolByNamespace(item)
			tool._rerender()
		else:
			raise ValueError('%s is an unknown qualifier' % qual)

	# this method triggers a backup of all installed tools supporting
	# backups to the cloud. It returns a list of created files
	def backup(self):
		now = time.time()
		macros = self.getToolByNamespace('macros')
		prefs = self.getPreferencesTool()
		base = _PyDS.HOMEDIR
		if prefs.get('private','upstreambackup'):
			base = _PyDS.ROOTDIR
		l = []
		for tool in PyDS.Tool.getTools():
			if hasattr(tool, '_backup'):
				l.extend(tool._backup())
		xml = '<files>\n'
		for fn in l:
			st = os.stat(fn)
			size = st[stat.ST_SIZE]
			short = fn
			if len(fn) > len(base):
				if fn[:len(base)] == base:
					short = fn[len(base):]
			if short[0] == '/':
				short = short[1:]
			xml += ' <file path="%s" created="%s" size="%d"/>\n' % (
				short.replace('\\','/'),
				macros.TodayFull(now=now),
				size
			)
		xml += '</files>\n'
		l.append(
			self.storeXML(
				xml,
				cloud=prefs.get('private', 'upstreambackup'),
				name='files.xml',
				toolname='backup'
			)
		)
		return l
	
	# this method returns the nugget in the menu on the right
	def nugget(self, url=None):
		pref = self.getPreferencesTool()
		html = PyDS.Tool.StandardTool.nugget(self, url)
		if self.get('prefs','doupstream') == 0:
			html += '<br><br><span class="errormsg">%s</span>' % (
				_('Upstreaming is disabled!'),
			)
		elif pref.offline():
			html += '<br><br><span class="errormsg">%s</span>' % (
				_('System is offline!'),
			)
		return html

	# this just passes through to SOAPpy.SOAPProxy and returns a
	# server for SOAP calls. Use the same parameters as you would with
	# SOAPProxy. If a http_proxy is defined, it is patched into **kw,
	# so this will return a SOAPProxy that will always work.
	def getServerForSOAP(self, *args, **kw):
		try:
			self._acquire()
			if not(kw.has_key('useproxy')):
				kw['useproxy'] = 1
			if self.prefs[0].proxy and kw['useproxy']:
				kw['http_proxy'] = self.prefs[0].proxy
			del kw['useproxy']
			return apply(SOAPpy.SOAPProxy, args, kw)
		finally: self._release()

	# this returns a server for XML-RPC calls that uses your
	# proxy settings and gives PyDS as the user agent header.
	def getServerForXMLRPC(self, url, useproxy=1):
		try:
			self._acquire()
			server = None
			transport = PatchingTransport()
			if self.prefs[0].proxy and useproxy:
				transport = ProxiedTransport()
				transport.set_proxy(self.prefs[0].proxy)
			return xmlrpclib.Server(url, transport=transport)
		finally: self._release()
	
	# this method awakens the upstream process on it's next check interval.
	# usefull if you generate content that should be upstreamed as fast
	# as possible. This usually is called by store methods, so you only
	# need to call it if you generate content externally or don't use
	# the store methods.
	def awakeUpstream(self):
		flag = self._awakeUpstream
		self._awakeUpstream = 1
		return flag

	# this method tells the server that upstreaming needs some
	# mandatory preferences.
	def needsStuff(self, req=None):
		return self._driver.needsStuff(req)
				
	# this method hooks up a callback for upstreams of URLs that end on
	# suffix.  the hook is called with the url and the content of the just
	# upstreamed file. The hook should return a set of (filename, contents)
	# pairs to upstream. The filename of that result set must be relative
	# to the upstream cache.
	def hookURLSuffix(self, suffix, level, hook):
		tool = hook.im_self.name
		func = hook.im_func.func_name
		if self.hooks.has_key(suffix):
			found = 0
			for hook in self.hooks[suffix]:
				if not(found) and hook == (level, tool, func):
					found = 1
			if not(found):
				self.hooks[suffix].append((level, tool, func))
			self.hooks[suffix].sort(lambda a,b: cmp(a[0],b[0]))
		else:
			self.hooks[suffix] = [(level, tool, func)]
			
	# checks wether a given URL is hooked and returns all hooks that hook
	def getHooksForURL(self, url):
		hooks = []
		for hook in self.hooks.keys():
			try:
				if url[-len(hook):] == hook:
					hooks.extend(self.hooks[hook])
			except: pass
			hooks.sort(lambda a,b: cmp(a[0],b[0]))
		return map(lambda row: self.getMethodByNamespace(row[1], row[2]), hooks)

	def index_html(self, req):

		try:
			self._acquire()
			refresh = req.getFirstValue('refresh')
			macros = self.getToolByNamespace('macros')
			pref = self.getPreferencesTool()
			html = PyDS.Tool.StringCollector()
			html += _('<h2>Status from the Community Server</h2>')
			if self.get('prefs','doupstream') == 0:
				html += '<p class="errormsg">%s</p>' % (
					_('Upstreaming is currently disabled! Enable it in the preferences!')
				)
			elif pref.offline():
				html += '<p class="errormsg">%s</p>' % (
					_('System is currently in offline mode!')
				)
			if (self.get('prefs','doupstream') != 0 or refresh) and pref.online():
				self.getInitialResources()
				self.getServerCapabilities()
				self.pingCloud(1)
			prefs = self.prefs[0]
			status = self.status[0]
			vtable = (
				(_('Driver used'), self._driver.title),
				(_('Member since'), status.membersince),
				(_('Last Ping'), status.lastping),
				(_('Bytes used'), macros.formatFileSize(status.bytesinuse)),
				(_('Bytes available'), macros.formatFileSize(prefs.maxbytes - status.bytesinuse)),
				(_('Files upstreamed'), status.upstreams),
				(_('Files deleted'), status.deletes),
				(_('Bytes upstreamed'), macros.formatFileSize(status.bytesupstreamed)),
				(_('Last Upstream'), status.lastupstream),
			)
			html += '<ul>'
			for t in vtable:
				html += '<li><b>%s</b>: %s</li>' % t
			html += '</ul>'
			if pref.offline():
				if self.get('prefs','doupstream') == 0:
					html += '<p><a href="%s?refresh=1">%s</a>' % (
						req.getUrl(),
						_('Click here to refresh data from the community server.')
					)
			html += _('<h2>Set of pending changes</h2>')
			html += '<p><a href="%s">%s</a>' % (
				req.getUrl('reupstream_redir'),
				_('Click here to force new upstreaming of all your files')
			)
			html += '<table border=0 width="100%">'
			tosave, todel = self.getChangeSet()
			for f in tosave:
				html += '<tr>'
				html += '<td class="whiteboxsmall">%s</td>' % f[0]
				html += _('<td class="whiteboxsmall">save %s</td>') % macros.formatFileSize(f[1])
				html += '</tr>'
			for f in todel:
				html += '<tr>'
				html += '<td class="whiteboxsmall">%s</td>' % f[0]
				html += _('<td class="whiteboxsmall">delete %s</td>') % macros.formatFileSize(f[1])
				html += '</tr>'
			if (len(todel) or len(tosave)) and pref.online():
				html += '<tr><td colspan="2" align="right">'
				html += '<form action="%s">' % (
					self.getUrl('awake_redir')
				)
				html += '<input type=submit name=UpstreamSubmit value="%s">' % _('upstream now')
				html += '</form>'
				html += '</td></tr>'
			html += '</table>'
			html += _('<h2>Tools that support rerendering</h2>')
			html += _('<p>Tag all tools whose contents you want to be rerendered and press the rerender button.')
			html += '<form action="%s" method="POST">' % self.getUrl('rerender_redir')
			html += '<table border=0 width="100%">'
			for tool in PyDS.Tool.getTools():
				if hasattr(tool, '_rerender'):
					html += '<tr>'
					html += '<td><input type="checkbox" name="tool" value="%s"></td>' % tool.name
					html += '<td><b>%s</b>: %s</td>' % (tool.title, tool.desc)
					html += '</tr>'
			html += '<tr><td colspan="2" align="right">'
			html += '<input type=submit name=RerenderSubmit value="%s">' % _('rerender')
			html += '</td></tr>'
			html += '</table>'
			html += '</form>'
			html += _('<h2>Important server URLs</h2>')
			html += '<ul>'
			if self.get('prefs', 'mailtourl'):
				html += '<li><b>%s:</b> %s</li>' % (_('Spamfree mailto'), self.get('prefs','mailtourl'))
			if self.get('prefs', 'webbugurl'):
				html += '<li><b>%s:</b> %s</li>' % (_('Webbug'), self.get('prefs','webbugurl'))
			if self.get('prefs','cancomment'):
				html += '<li><b>%s:</b> %s</li>' % (_('Comments CGI'), self.get('prefs','commentsurl'))
			if self.get('prefs','cantrackback'):
				html += '<li><b>%s:</b> %s</li>' % (_('Trackback CGI'), self.get('prefs','trackbackurl'))
			if self.get('prefs','hassearch'):
				html += '<li><b>%s:</b> %s</li>' % (_('Search CGI'), self.get('prefs','urlsearch'))
			html += '</ul>'
			req.setLocalValue('title', _('Upstreaming Status'))
			return req.renderPage('BaseTemplate', body=str(html))
		finally: self._release()

	def rerender_redir(self, req):
		tools = req.getAllValues('tool')
		for toolname in tools:
			tool = self.getToolByNamespace(toolname)
			tool._rerender()
		return req.getUrl()
	
	def awake_redir(self, req):
		self.awakeUpstream()
		return req.getUrl()
	
	def reupstream_redir(self, req):
		try:
			self._acquire()
			l = range(len(self.filestatus))
			l.reverse()
			for idx in l:
				self.filestatus.delete(idx)
			self._commit()
		finally: self._release()
		self.awakeUpstream()
		return req.getUrl()
	
	# this method returns descriptions of hooked up preferences panels
	# for this tool
	def getPreferencePanels(self):
		return [
			['upstream_panel', _('Upstreaming'),
				_('Here you can configure your community server you want to upstream to.')]
		]
			
	def upstream_panel(self, req):
		return self._driver.upstream_panel(req)
		
	def upstream_redir(self, req):
		return self._driver.upstream_redir(req)

	# this calls the radioCommunityServer.getInitialResource via XML-RPC
	# and stores the returned data into the database
	def getInitialResources(self):
		return self._driver.getInitialResources()

	# this calls the xmlStorageSystem.getServerCapabilities via XML-RPC
	# and stores the returned data into the database
	def getServerCapabilities(self):
		return self._driver.getServerCapabilities()

	# this calls the xmlStorageSystem.ping via XML-RPC
	# and stores the returned data into the database. This tells
	# the server wether you are online or not.
	def pingCloud(self, online=1, category=''):
		return self._driver.pingCloud(online, category)
	
	# this method returns the list of all files in the filestatus database
	def getFilestatusList(self):
		try:
			self._acquire()
			macros = self.getToolByNamespace('macros')
			res = []
			for file in self.filestatus:
				res.append({
					'path': file.path,
					'time': macros.TodayFull(now=file.time),
					'size': file.size
				})
			return res
		finally: self._release()
	
	# this method returns the status of a file with given path
	def getFilestatus(self, path):
		try:
			self._acquire()
			macros = self.getToolByNamespace('macros')
			(idx, found) = self.filestatus.locate({'path':path})
			if found:
				file = self.filestatus[idx]
				return {
					'path': file.path,
					'time': macros.TodayFull(now=file.time),
					'size': file.size
				}
			else:
				return {}
		finally: self._release()

	# this method returns the list of files in ROOTDIR that have changed
	# since last successfull upstream. Returned is a list of files to
	# save and a list of files to delete
	def getChangeSet(self):

		files = {}
		fstat = {}

		def getDirectory(fullpath, path=None):
			for file in os.listdir(fullpath):
				if file[0] != '.':
					fname = file
					if path:
						fname = os.path.join(path, file)
					fullname = os.path.join(fullpath, file)
					if os.path.isdir(fullname):
						getDirectory(fullname, fname)
					else:
						st = os.stat(fullname)
						files[fname] = (st[6], st[8])
						
		try:
			self._acquire()
			for f in self.filestatus:
				fstat[f.path] = [f.size, f.time]
		finally: self._release()
	
		getDirectory(_PyDS.ROOTDIR)
		
		todel = []
		tosave = []
		for fn in files.keys():
			if fstat.has_key(fn):
				if fstat[fn][0] != files[fn][0]:
					tosave.append((fn, files[fn][0]))
				elif fstat[fn][1] < files[fn][1]:
					tosave.append((fn, files[fn][0]))
			else:
				tosave.append((fn, files[fn][0]))
		for fn in fstat.keys():
			if not(files.has_key(fn)):
				todel.append((fn, fstat[fn][0]))

		return (tosave, todel)

	# this method deletes multiple files on the cloud
	def deleteMultipleFiles(self, files):
		return self._driver.deleteMultipleFiles(files)
			
	# this method upstreams files with given contents without keeping track of
	# their status (actually without the need of them to be in the filesystem!)
	def upstreamMultipleFiles(self, files, contents):
		return self._driver.upstreamMultipleFiles(files, contents)

	# this method copies files to the cloud and gives you the addresses
	# of upstreamed files
	def saveMultipleFiles(self, files):
		contents = []
		for fn in files:
			try:
				f = open(os.path.join(_PyDS.ROOTDIR, fn), 'rb')
				content = f.read()
				contents.append(content)
				f.close()
			except:
				contents.append('')
		res = self.upstreamMultipleFiles(files, contents)
		if not(res.get('flError')):
			try:
				self._acquire()
				for i in range(len(files)):
					url = res['urlList'][i]
					hooks = self.getHooksForURL(url)
					if hooks:
						ftab = []
						for hook in hooks:
							ftab.extend(hook(url, contents[i]))
						if len(ftab):
							ftabfiles = map(lambda row: row[0], ftab)
							ftabcontents = map(lambda row: row[1], ftab)
							ftabres = self.upstreamMultipleFiles(ftabfiles, ftabcontents)
							for i in range(len(ftabfiles)):
								self.log(
									_('upstreamed dependend file %s'),
									'<a href="%s">%s</a>' % (
										ftabres['urlList'][i],
										files[i]
									)
								)
					self.log(
						_('upstreamed file %s'),
						'<a href="%s" target="_blank">%s</a>' % (
							url,
							files[i]
						)
					)
					st = os.stat(files[i])
					size = st[6]
					time = st[8]
					(fnr, found) = self.filestatus.locate({'path':files[i]})
					if not(found):
						self.filestatus.append(
							path=files[i],
							time=time,
							size=size
						)
					else:
						self.filestatus[fnr].time = time
						self.filestatus[fnr].size = size
			finally:
				self._commit()
				self._release()
			return res['urlList']

	# this method returns a stored status or prefs value
	def get(self, table, field):
		if table == 'prefs':
			if field == 'proxy' and self.prefs[0].proxy == '':
				return _PyDS.proxy
			return getattr(self.prefs[0], field, '')
		elif table == 'status':
			return getattr(self.status[0], field, '')
		elif table == 'driver' and len(self.driver):
			return getattr(self.driver[0], field, '')
	
	# this method sets a preference or status value. It returs the old
	# value of the field
	def set(self, table, field, value):
		try:
			self._acquire()
			old = ''
			if table == 'prefs':
				if hasattr(self.prefs[0], field):
					old = getattr(self.prefs[0], field)
					setattr(self.prefs[0], field, value)
			elif table == 'status':
				if hasattr(self.status[0], field):
					old = getattr(self.status[0], field)
					setattr(self.status[0], field, value)
			elif table == 'driver' and len(self.driver):
				if hasattr(self.driver[0], field):
					old = getattr(self.driver[0], field)
					setattr(self.driver[0], field, value)
			self._commit()
			return old
		finally: self._release()
	
	# this method returns an URL for the upstream_panel with attached
	# error message (this is used in drivers)
	def panelErrorUrl(self, req, errmsg):
		return req.getUrl(
			'panel_html',
			tool='preferences',
			panel='upstream_panel',
			fortool='upstream',
			errmsg=errmsg
		)

	# this method returns an URL for the upstream_panel with attached
	# information message (this is used in drivers)
	def panelInfoUrl(self, req, infomsg):
		return req.getUrl(
			'panel_html',
			tool='preferences',
			panel='upstream_panel',
			fortool='upstream',
			infomsg=infomsg
		)

	# this method returns an URL for the upstream_panel
	def panelUrl(self, req):
		return req.getUrl(
			'panel_html',
			tool='preferences',
			panel='upstream_panel',
			fortool='upstream'
		)
	
	# this method deletes a file from the filestatus. This respects
	# the fact that once the filestatus database wasn't ordered and so
	# searches first binary and then sequential
	def deleteFileStatus(self, file):
		try:
			self._acquire()
			(idx, found) = self.filestatus.locate({'path':file})
			if found:
				self.filestatus.delete(idx)
			else:
				idx = 0
				for file in self.filestatus:
					if file.path == path:
						self.filestatus.delete(idx)
						idx += 1
			self._commit()
		finally: self._release()
	

__desc__ = _("This tool allows to configure the community server you want to upstream to.")

PyDS.Tool.registerTool(UpstreamTool, 'upstream', _('Upstreaming'), __desc__, 910)

