"""

Python Desktop Server - Upstreaming Driver for XMLStorageSystem

Copyright (c) 2002, Georg Bauer <gb@murphy.bofh.ms>

Permission is hereby granted, free of charge, to any person obtaining a copy of 
this software and associated documentation files (the "Software"), to deal in 
the Software without restriction, including without limitation the rights to 
use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies of
 
the Software, and to permit persons to whom the Software is furnished to do so, 
subject to the following conditions:

The above copyright notice and this permission notice shall be included in all 
copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR 
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS
 
FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR 
COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER 
IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN 
CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.

"""

# $Id: XSSDriver.py,v 1.6 2003/12/30 21:12:27 gb Exp $

import md5
import xmlrpclib

from PyDS.DriverRegistry import registerDriver, BaseDriver

class XSSDriver(BaseDriver):

	def initopts(self):
		tool = self.tool
		tool.server = None
		if tool.get('prefs', 'rpcurl'):
			tool.server = tool.getServerForXMLRPC(tool.get('prefs', 'rpcurl'))

	def _backup(self):
		return []

	# this method tells the server that upstreaming needs some
	# mandatory preferences.
	def needsStuff(self, req=None):
		if type(self.tool.server) == type(None):
			if req:
				return self.tool.panelErrorUrl(
					req,
					_('You need to give your upstream server XML-RPC url and your password')
				)
			else:
				return 1
		else:
			return 0
				
	def upstream_panel(self, req):
		tool = self.tool
		macros = tool.getToolByNamespace('macros')
		html = '<p>%s' % _("Here you can configure your community server you want to upstream to.")
		html += '<form action="%s" method="POST">' % req.getUrl('upstream_redir', tool='upstream')
		html += '<table border=0 width="100%">'
		html += '<tr><td colspan=2>%s</td></tr>' % _("Your XML-RPC url is usually of the form http://muensterland.org/RPC2 - ask your administrator if you are unsure about that")
		html += '<tr><td>%s</td><td>' %_('XML-RPC url')
		html += '<input name="rpcurl" size=50 value="%s">' % tool.get('prefs', 'rpcurl')
		html += '</td></tr><tr><td>%s</td><td>' % _('enter password')
		html += '<input type=password name="password" size=20>'
		html += '</td></tr><tr><td>%s</td><td>' % _('repeat password')
		html += '<input type=password name="password2" size=20>'
		html += '</td></tr><tr><td>%s</td><td>' % _('proxy server')
		html += '<input name="proxy" size=20 value="%s">' % tool.get('prefs', 'proxy')
		html += '</td></tr><tr><td>%s</td><td>%s' % (_('Usernumber'), tool.get('prefs', 'usernum'))
		html += '</td></tr><tr><td>%s</td><td>' % _('ping enabled')
		html += macros.checkBox('doping', tool.get('prefs', 'doping'))
		html += '</td></tr><tr><td>%s</td><td>' % _('ping interval')
		html += '<input name="pinginterval" size=5 value="%d">' % tool.get('prefs', 'pinginterval')
		html += '</td></tr><tr><td>%s</td><td>' % _('upstream enabled')
		html += macros.checkBox('doupstream', tool.get('prefs', 'doupstream'))
		html += '</td></tr><tr><td>%s</td><td>' % _('upstream interval')
		html += '<input name="upstreaminterval" size=5 value="%d">' % tool.get('prefs', 'upstreaminterval')
		html += '</td></tr><tr><td colspan=2 align="right">'
		html += '<input type=submit name="UpstreamSubmit" value="%s">' % _('save')
		html += '</td></tr>'
		html += '</table>'
		html += '</form>'
		return html
		
	def upstream_redir(self, req):
		tool = self.tool
		rpcurl = req.getFirstValue('rpcurl')
		password = req.getFirstValue('password')
		password2 = req.getFirstValue('password2')
		proxy = req.getFirstValue('proxy')
		if password != password2:
			return tool.panelErrorUrl(
				req,
				_("You didn't enter the password exactly the same in both fields")
			)
		tool.set('prefs', 'rpcurl', rpcurl)
		tool.set('prefs', 'proxy', proxy)
		try:
			tool.set('prefs', 'pinginterval', int(req.getFirstValue('pinginterval',900)))
			tool.set('prefs', 'upstreaminterval', int(req.getFirstValue('upstreaminterval',900)))
		except: pass
		try:
			doupstream = int(req.getFirstValue('doupstream',0))
		except:
			doupstream = 0
		try:
			doping = int(req.getFirstValue('doping'))
		except:
			doping = 0
		doupstreamold = tool.get('prefs', 'doupstream')
		tool.set('prefs', 'doupstream', doupstream)
		tool.set('prefs', 'doping', doping)
		tool._initopts()
		if doupstreamold != doupstream and doupstream:
			tool.awakeUpstream()
		if password:
			password = md5.md5(password).hexdigest()
			prefs = tool.getToolByNamespace('preferences')
			res = tool.server.xmlStorageSystem.registerUser(
				prefs.get('private','email'),
				prefs.get('private','name'),
				password,
				_PyDS.serverport,
				'Python Desktop Server'
			)
			if res['flError']:
				return tool.panelErrorUrl(
					req,
					_("The Community Server says: %s") % res.get('message',_('Nothing'))
				)
			tool.set('prefs', 'usernum', res['usernum'])
			tool.set('prefs', 'password', password)
			self.getInitialResources()
			self.getServerCapabilities()
			self.pingCloud(1)
			return tool.panelInfoUrl(
				req,
				_("The Community Server says: %s") % res.get('message',_('Nothing'))
			)
		else:
			return tool.panelUrl(req)

	# this calls the radioCommunityServer.getInitialResource via XML-RPC
	# and stores the returned data into the database
	def getInitialResources(self):
		tool = self.tool
		res = tool.server.radioCommunityServer.getInitialResources()
		tool.set('prefs', 'commentsurl', res.get('commentsPageUrl',''))
		tool.set('prefs', 'trackbackurl', res.get('trackbackPageUrl',''))
		return res

	# this calls the xmlStorageSystem.getServerCapabilities via XML-RPC
	# and stores the returned data into the database
	def getServerCapabilities(self):
		tool = self.tool
		res = tool.server.xmlStorageSystem.getServerCapabilities(
			tool.get('prefs', 'usernum'),
			tool.get('prefs', 'password')
		)
		if res['flError']:
			tool.log('getServerCapabilities gave %s', res['message'])
		else:
			if res['community'].get('flCanHostComments',0):
				tool.set('prefs', 'cancomment', 1)
			if res['community'].get('flCanHostTrackback',0):
				tool.set('prefs', 'cantrackback', 1)
			if res['community'].get('flCanMirrorPosts',0):
				tool.set('prefs', 'canmirrorposts', 1)
			if res['community'].get('flHasSearch',0):
				tool.set('prefs', 'hassearch', 1)
				tool.set('prefs', 'urlsearch', res['community'].get('urlSearch', ''))
			tool.set('prefs', 'cloudurl', res['yourUpstreamFolderUrl'])
			tool.set('prefs', 'bytesused', res['ctBytesInUse'])
			tool.set('prefs', 'maxbytes', res['maxBytesPerUser'])
			tool.set('prefs', 'maxfilesize', res['maxFileSize'])
			tool.set('prefs', 'rankingurl', res['urlRankingsByPageReads'])
			tool.set('prefs', 'refererurl', res['urlReferers'])
			tool.set('prefs', 'mailtourl', res['urlSpamFreeMailto'])
			tool.set('prefs', 'updatesurl', res['urlWeblogUpdates'])
			tool.set('prefs', 'webbugurl', res['webBugUrl'])
		return res

	# this calls the xmlStorageSystem.ping via XML-RPC
	# and stores the returned data into the database. This tells
	# the server wether you are online or not.
	def pingCloud(self, online=1, category=''):
		tool = self.tool
		prefs = tool.getToolByNamespace('preferences')
		weblog = tool.getToolByNamespace('weblog')
		userinfo = {}
		if online:
			userinfo = {
				'email':prefs.get('private','email'),
				'serialNumber':_('PyDS is free software!'),
				'organization':'',
				'flBehindFirewall':tool.get('prefs','proxy') != '',
				'name':prefs.get('private','name')
			}
			if category:
				userinfo['weblogTitle'] = weblog.getCategory(category)['title']
			else:
				userinfo['weblogTitle'] = prefs.get('private','title')
		res = self.tool.server.xmlStorageSystem.ping(
			tool.get('prefs', 'usernum'),
			tool.get('prefs', 'password'),
			online,
			_PyDS.serverport,
			userinfo
		)
		if res['flError']:
			tool.log('ping gave %s', res['message'])
		else:
			cloud = res['cloudData']
			tool.set('status', 'email', cloud['email'])
			tool.set('status', 'name', cloud['name'])
			tool.set('status', 'organization', cloud['organization'])
			tool.set('status', 'deletes', cloud['ctFileDeletions'])
			tool.set('status', 'membersince', cloud['whenCreated'])
			tool.set('status', 'pings', cloud['ctAccesses'])
			tool.set('status', 'serial', cloud['serialNumber'])
			tool.set('status', 'url', cloud['url'])
			tool.set('status', 'behindwall', cloud['flBehindFirewall'])
			tool.set('status', 'title', cloud['weblogTitle'])
			tool.set('status', 'upstreams', cloud['ctUpstreams'])
			tool.set('status', 'signons', cloud['ctSignons'])
			tool.set('status', 'lastping', cloud['whenLastAccess'])
			tool.set('status', 'lastupstream', cloud['whenLastUpstream'])
			tool.set('status', 'port', cloud['port'])
			tool.set('status', 'bytesupstreamed', cloud['ctBytesUpstreamed'])
			tool.set('status', 'bytesinuse', cloud['ctBytesInUse'])
			tool.set('status', 'lastsignoff', cloud['whenLastSignOff'])
			tool.set('status', 'lastsignon', cloud['whenLastSignOn'])
			tool.set('status', 'signedon', cloud['flSignedOn'])
			tool.set('status', 'usernum', cloud['usernum'])
			tool.set('prefs', 'cloudurl', res['yourUpstreamFolderUrl'])
		return res
	
	# this method deletes multiple files on the cloud
	def deleteMultipleFiles(self, files):
		tool = self.tool
		res = tool.server.xmlStorageSystem.deleteMultipleFiles(
			tool.get('prefs', 'usernum'),
			tool.get('prefs', 'password'),
			map(lambda fn: fn.replace('\\', '/'), files)
		)
		for path in files:
			tool.deleteFileStatus(path)
		if res['flError']:
			tool.log(_('Error on upstream delete: %s'), res['message'])
		else:
			tool.log(_('Message on upstream delete: %s'), res['message'])
		return res
			
	# this method upstreams files with given contents without keeping track of
	# their status (actually without the need of them to be in the filesystem!)
	def upstreamMultipleFiles(self, files, contents):
		tool = self.tool
		res = tool.server.xmlStorageSystem.saveMultipleFiles(
			tool.get('prefs','usernum'),
			tool.get('prefs','password'),
			map(lambda fn: fn.replace('\\', '/'), files),
			map(lambda txt: xmlrpclib.Binary(txt), contents)
		)
		if res.get('flError'):
			tool.log(_('Error on upstream: %s'), res['message'])
		else:
			tool.log(_('Message on upstream: %s'), res['message'])
		return res

registerDriver('XSS', _('XML Storage System (default driver)'), XSSDriver)

